using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.Security.Principal;
using System.Text.RegularExpressions;
using System.IO;
using com.ibm.gre;

namespace SampleApp
{
	/// <summary>
	/// Form1 is the main form for the application.
	/// It contains three tabbed pages.
	/// The first page diplays the File Plan.
	/// The second page displays Life Cycle batch operation results.
	/// The third page displays logging and error information.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		/// <summary>
		/// Constructor:
		/// <p>Configuration setting are read from the app.config file.</p>
		/// <p>The value Form1.m_isHostLogin determines if the login or hostLogin method will be used
		/// to authenticate with the Records Manager.</p>
		/// <p>The value Form1.m_serverNamePort specifies the Records Manager Web Services
		/// server name and port.</p>
		/// <p>The value Form1.m_databaseName specifies the Records Manager databa source to use.</p>
		/// <p>Form1.m_language specifies the language to use.</p>
		/// <p>Form1.m_userName specifies the user name to use for logging in.</p>
		/// </summary>
		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			util = new Util(this,this);

			System.Configuration.AppSettingsReader configurationAppSettings = 
			new System.Configuration.AppSettingsReader();
			//string temp 
			this.m_isHostLogin = ((bool)(configurationAppSettings.GetValue("Form1.m_isHostLogin",typeof(bool))));
			this.m_serverNamePort = ((string)(configurationAppSettings.GetValue("Form1.m_serverNamePort",typeof(string))));
			this.m_databaseName = ((string)(configurationAppSettings.GetValue("Form1.m_databaseName",typeof(string))));
			this.m_language = ((string)(configurationAppSettings.GetValue("Form1.m_language",typeof(string))));
			this.m_userName = ((string)(configurationAppSettings.GetValue("Form1.m_userName",typeof(string))));
			WindowsIdentity me = WindowsIdentity.GetCurrent();
			m_extUserId = me.Name;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Form1));
			System.Configuration.AppSettingsReader configurationAppSettings = new System.Configuration.AppSettingsReader();
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.mnuLogin = new System.Windows.Forms.MenuItem();
			this.mnuHostLogin = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.mnuTestCode = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.mnuJavadocsLocation = new System.Windows.Forms.MenuItem();
			this.mnuUpdateAttributeTypeFile = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.mnuAttrCntrlMaskInterigator = new System.Windows.Forms.MenuItem();
			this.openFileDialog1 = new System.Windows.Forms.OpenFileDialog();
			this.cntxtMnuTreeView = new System.Windows.Forms.ContextMenu();
			this.mnuAddTreeItemTarget = new System.Windows.Forms.MenuItem();
			this.mnuEditTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuViewTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuDeleteTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuRefreshTreeItem = new System.Windows.Forms.MenuItem();
			this.menuItem5 = new System.Windows.Forms.MenuItem();
			this.mnuMetadataTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuUserPermissionsTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuExtendedPermissionsTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuMetadaPlus = new System.Windows.Forms.MenuItem();
			this.menuItem8 = new System.Windows.Forms.MenuItem();
			this.mnuSearchTreeItem = new System.Windows.Forms.MenuItem();
			this.menuLifeCycleQuery = new System.Windows.Forms.MenuItem();
			this.menuItem6 = new System.Windows.Forms.MenuItem();
			this.mnuAutoGen = new System.Windows.Forms.MenuItem();
			this.cntxtMnuListView = new System.Windows.Forms.ContextMenu();
			this.mnuAddListItemTarget = new System.Windows.Forms.MenuItem();
			this.mnuEditListItem = new System.Windows.Forms.MenuItem();
			this.mnuViewListItem = new System.Windows.Forms.MenuItem();
			this.mnuDeleteListItem = new System.Windows.Forms.MenuItem();
			this.menuItem9 = new System.Windows.Forms.MenuItem();
			this.mnuMigrateLitsItems = new System.Windows.Forms.MenuItem();
			this.mnuUnMigrateListItems = new System.Windows.Forms.MenuItem();
			this.tabControl2 = new System.Windows.Forms.TabControl();
			this.tabFilePlanExplorer = new System.Windows.Forms.TabPage();
			this.lvComponents = new System.Windows.Forms.ListView();
			this.columnHeaderComponent = new System.Windows.Forms.ColumnHeader();
			this.columnHeaderName = new System.Windows.Forms.ColumnHeader();
			this.columnHeaderTitle = new System.Windows.Forms.ColumnHeader();
			this.imageList1 = new System.Windows.Forms.ImageList(this.components);
			this.splitter2 = new System.Windows.Forms.Splitter();
			this.tvFilePlan = new System.Windows.Forms.TreeView();
			this.splitter1 = new System.Windows.Forms.Splitter();
			this.tabLifeCycleReports = new System.Windows.Forms.TabPage();
			this.lvLifeCycleReports = new System.Windows.Forms.ListView();
			this.cntxtMnuBatchList = new System.Windows.Forms.ContextMenu();
			this.mnuViewLifeCycleResults = new System.Windows.Forms.MenuItem();
			this.mnuTransitionLifeCycleResults = new System.Windows.Forms.MenuItem();
			this.menuViewTransitionResults = new System.Windows.Forms.MenuItem();
			this.mnuDeleteLifeCycleResults = new System.Windows.Forms.MenuItem();
			this.tabTextPage = new System.Windows.Forms.TabPage();
			this.textBox1 = new System.Windows.Forms.TextBox();
			this.statusBar1 = new System.Windows.Forms.StatusBar();
			this.folderBrowserDialog1 = new System.Windows.Forms.FolderBrowserDialog();
			this.menuItem7 = new System.Windows.Forms.MenuItem();
			this.mnuTemplNoneTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuTemplExpandLstTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuTemplProfilesTreeItem = new System.Windows.Forms.MenuItem();
			this.mnuTemplListsProfilesTreeItem = new System.Windows.Forms.MenuItem();
			this.tabControl2.SuspendLayout();
			this.tabFilePlanExplorer.SuspendLayout();
			this.tabLifeCycleReports.SuspendLayout();
			this.tabTextPage.SuspendLayout();
			this.SuspendLayout();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.menuItem1,
																					  this.menuItem3});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuLogin,
																					  this.mnuHostLogin});
			this.menuItem1.Text = "File";
			// 
			// mnuLogin
			// 
			this.mnuLogin.Index = 0;
			this.mnuLogin.Text = "Login";
			this.mnuLogin.Click += new System.EventHandler(this.mnuLogin_Click);
			// 
			// mnuHostLogin
			// 
			this.mnuHostLogin.Index = 1;
			this.mnuHostLogin.Text = "Host Login";
			this.mnuHostLogin.Click += new System.EventHandler(this.mnuHostLogin_Click);
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 1;
			this.menuItem3.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuTestCode,
																					  this.menuItem2,
																					  this.mnuJavadocsLocation,
																					  this.mnuUpdateAttributeTypeFile,
																					  this.menuItem4,
																					  this.mnuAttrCntrlMaskInterigator});
			this.menuItem3.Text = "Tools";
			// 
			// mnuTestCode
			// 
			this.mnuTestCode.Index = 0;
			this.mnuTestCode.Text = "TestCode";
			this.mnuTestCode.Click += new System.EventHandler(this.mnuTestCode_Click);
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 1;
			this.menuItem2.Text = "-";
			// 
			// mnuJavadocsLocation
			// 
			this.mnuJavadocsLocation.Index = 2;
			this.mnuJavadocsLocation.Text = "Javadocs Location";
			this.mnuJavadocsLocation.Click += new System.EventHandler(this.mnuJavadocsLocation_Click);
			// 
			// mnuUpdateAttributeTypeFile
			// 
			this.mnuUpdateAttributeTypeFile.Index = 3;
			this.mnuUpdateAttributeTypeFile.Text = "Update Attribute Type File";
			this.mnuUpdateAttributeTypeFile.Click += new System.EventHandler(this.mnuUpdateAttributeTypeFile_Click);
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 4;
			this.menuItem4.Text = "-";
			// 
			// mnuAttrCntrlMaskInterigator
			// 
			this.mnuAttrCntrlMaskInterigator.Index = 5;
			this.mnuAttrCntrlMaskInterigator.Text = "AttrCntrlMask Interigator";
			this.mnuAttrCntrlMaskInterigator.Click += new System.EventHandler(this.mnuAttrCntrlMaskInterigator_Click);
			// 
			// cntxtMnuTreeView
			// 
			this.cntxtMnuTreeView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							 this.mnuAddTreeItemTarget,
																							 this.mnuEditTreeItem,
																							 this.mnuViewTreeItem,
																							 this.mnuDeleteTreeItem,
																							 this.mnuRefreshTreeItem,
																							 this.menuItem5,
																							 this.menuItem7,
																							 this.menuItem8,
																							 this.mnuSearchTreeItem,
																							 this.menuLifeCycleQuery,
																							 this.menuItem6,
																							 this.mnuAutoGen});
			this.cntxtMnuTreeView.Popup += new System.EventHandler(this.cntxtMnuTreeView_Popup);
			// 
			// mnuAddTreeItemTarget
			// 
			this.mnuAddTreeItemTarget.Index = 0;
			this.mnuAddTreeItemTarget.Text = "Add";
			this.mnuAddTreeItemTarget.Click += new System.EventHandler(this.mnuAddTreeItemTarget_Click);
			// 
			// mnuEditTreeItem
			// 
			this.mnuEditTreeItem.Index = 1;
			this.mnuEditTreeItem.Text = "Edit";
			this.mnuEditTreeItem.Click += new System.EventHandler(this.mnuEditTreeItem_Click);
			// 
			// mnuViewTreeItem
			// 
			this.mnuViewTreeItem.Index = 2;
			this.mnuViewTreeItem.Text = "View";
			this.mnuViewTreeItem.Click += new System.EventHandler(this.mnuViewTreeItem_Click);
			// 
			// mnuDeleteTreeItem
			// 
			this.mnuDeleteTreeItem.Index = 3;
			this.mnuDeleteTreeItem.Text = "Delete";
			this.mnuDeleteTreeItem.Click += new System.EventHandler(this.mnuDeleteTreeItem_Click);
			// 
			// mnuRefreshTreeItem
			// 
			this.mnuRefreshTreeItem.Index = 4;
			this.mnuRefreshTreeItem.Text = "Refresh";
			this.mnuRefreshTreeItem.Click += new System.EventHandler(this.mnuRefreshTreeItem_Click);
			// 
			// menuItem5
			// 
			this.menuItem5.Index = 5;
			this.menuItem5.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuMetadataTreeItem,
																					  this.mnuUserPermissionsTreeItem,
																					  this.mnuExtendedPermissionsTreeItem,
																					  this.mnuMetadaPlus});
			this.menuItem5.Text = "XML";
			// 
			// mnuMetadataTreeItem
			// 
			this.mnuMetadataTreeItem.Index = 0;
			this.mnuMetadataTreeItem.Text = "Metadata";
			this.mnuMetadataTreeItem.Click += new System.EventHandler(this.mnuMetadataTreeItem_Click);
			// 
			// mnuUserPermissionsTreeItem
			// 
			this.mnuUserPermissionsTreeItem.Index = 1;
			this.mnuUserPermissionsTreeItem.Text = "User Permissions";
			this.mnuUserPermissionsTreeItem.Click += new System.EventHandler(this.mnuUserPermissionsTreeItem_Click);
			// 
			// mnuExtendedPermissionsTreeItem
			// 
			this.mnuExtendedPermissionsTreeItem.Index = 2;
			this.mnuExtendedPermissionsTreeItem.Text = "Extended Permissions";
			this.mnuExtendedPermissionsTreeItem.Click += new System.EventHandler(this.mnuExtendedPermissionsTreeItem_Click);
			// 
			// mnuMetadaPlus
			// 
			this.mnuMetadaPlus.Index = 3;
			this.mnuMetadaPlus.Text = "MetadataPlus";
			this.mnuMetadaPlus.Click += new System.EventHandler(this.mnuMetadaPlus_Click);
			// 
			// menuItem8
			// 
			this.menuItem8.Index = 7;
			this.menuItem8.Text = "-";
			// 
			// mnuSearchTreeItem
			// 
			this.mnuSearchTreeItem.Index = 8;
			this.mnuSearchTreeItem.Text = "Search...";
			this.mnuSearchTreeItem.Click += new System.EventHandler(this.mnuSearchTreeItem_Click);
			// 
			// menuLifeCycleQuery
			// 
			this.menuLifeCycleQuery.Index = 9;
			this.menuLifeCycleQuery.Text = "Life Cycle Query...";
			this.menuLifeCycleQuery.Click += new System.EventHandler(this.menuLifeCycleQuery_Click);
			// 
			// menuItem6
			// 
			this.menuItem6.Index = 10;
			this.menuItem6.Text = "-";
			// 
			// mnuAutoGen
			// 
			this.mnuAutoGen.Index = 11;
			this.mnuAutoGen.Text = "AutoGen";
			this.mnuAutoGen.Click += new System.EventHandler(this.mnuAutoGen_Click);
			// 
			// cntxtMnuListView
			// 
			this.cntxtMnuListView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							 this.mnuAddListItemTarget,
																							 this.mnuEditListItem,
																							 this.mnuViewListItem,
																							 this.mnuDeleteListItem,
																							 this.menuItem9,
																							 this.mnuMigrateLitsItems,
																							 this.mnuUnMigrateListItems});
			this.cntxtMnuListView.Popup += new System.EventHandler(this.cntxtMnuListView_Popup);
			// 
			// mnuAddListItemTarget
			// 
			this.mnuAddListItemTarget.Index = 0;
			this.mnuAddListItemTarget.Text = "Add";
			// 
			// mnuEditListItem
			// 
			this.mnuEditListItem.Index = 1;
			this.mnuEditListItem.Text = "Edit";
			this.mnuEditListItem.Click += new System.EventHandler(this.mnuEditListItem_Click);
			// 
			// mnuViewListItem
			// 
			this.mnuViewListItem.Index = 2;
			this.mnuViewListItem.Text = "View";
			this.mnuViewListItem.Click += new System.EventHandler(this.mnuViewListItem_Click);
			// 
			// mnuDeleteListItem
			// 
			this.mnuDeleteListItem.Index = 3;
			this.mnuDeleteListItem.Text = "Delete";
			this.mnuDeleteListItem.Click += new System.EventHandler(this.mnuDeleteListItem_Click);
			// 
			// menuItem9
			// 
			this.menuItem9.Index = 4;
			this.menuItem9.Text = "-";
			// 
			// mnuMigrateLitsItems
			// 
			this.mnuMigrateLitsItems.Index = 5;
			this.mnuMigrateLitsItems.Text = "Put in Container";
			this.mnuMigrateLitsItems.Click += new System.EventHandler(this.mnuMigrateListItems_Click);
			// 
			// mnuUnMigrateListItems
			// 
			this.mnuUnMigrateListItems.Index = 6;
			this.mnuUnMigrateListItems.Text = "Remove from Container";
			this.mnuUnMigrateListItems.Click += new System.EventHandler(this.mnuUnMigrateListItems_Click);
			// 
			// tabControl2
			// 
			this.tabControl2.Controls.Add(this.tabFilePlanExplorer);
			this.tabControl2.Controls.Add(this.tabLifeCycleReports);
			this.tabControl2.Controls.Add(this.tabTextPage);
			this.tabControl2.Dock = System.Windows.Forms.DockStyle.Fill;
			this.tabControl2.Location = new System.Drawing.Point(0, 0);
			this.tabControl2.Name = "tabControl2";
			this.tabControl2.SelectedIndex = 0;
			this.tabControl2.Size = new System.Drawing.Size(792, 585);
			this.tabControl2.TabIndex = 8;
			this.tabControl2.SelectedIndexChanged += new System.EventHandler(this.tabControl2_SelectedIndexChanged);
			// 
			// tabFilePlanExplorer
			// 
			this.tabFilePlanExplorer.Controls.Add(this.lvComponents);
			this.tabFilePlanExplorer.Controls.Add(this.splitter2);
			this.tabFilePlanExplorer.Controls.Add(this.tvFilePlan);
			this.tabFilePlanExplorer.Controls.Add(this.splitter1);
			this.tabFilePlanExplorer.Location = new System.Drawing.Point(4, 22);
			this.tabFilePlanExplorer.Name = "tabFilePlanExplorer";
			this.tabFilePlanExplorer.Size = new System.Drawing.Size(784, 559);
			this.tabFilePlanExplorer.TabIndex = 0;
			this.tabFilePlanExplorer.Text = "File Plan Explorer";
			// 
			// lvComponents
			// 
			this.lvComponents.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						   this.columnHeaderComponent,
																						   this.columnHeaderName,
																						   this.columnHeaderTitle});
			this.lvComponents.ContextMenu = this.cntxtMnuListView;
			this.lvComponents.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvComponents.FullRowSelect = true;
			this.lvComponents.Location = new System.Drawing.Point(198, 0);
			this.lvComponents.Name = "lvComponents";
			this.lvComponents.Size = new System.Drawing.Size(586, 559);
			this.lvComponents.SmallImageList = this.imageList1;
			this.lvComponents.TabIndex = 3;
			this.lvComponents.View = System.Windows.Forms.View.Details;
			// 
			// columnHeaderComponent
			// 
			this.columnHeaderComponent.Text = "Component";
			this.columnHeaderComponent.Width = 100;
			// 
			// columnHeaderName
			// 
			this.columnHeaderName.Text = "Name";
			this.columnHeaderName.Width = 100;
			// 
			// columnHeaderTitle
			// 
			this.columnHeaderTitle.Text = "Title";
			this.columnHeaderTitle.Width = 500;
			// 
			// imageList1
			// 
			this.imageList1.ImageSize = new System.Drawing.Size(16, 16);
			this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
			this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// splitter2
			// 
			this.splitter2.Location = new System.Drawing.Point(195, 0);
			this.splitter2.Name = "splitter2";
			this.splitter2.Size = new System.Drawing.Size(3, 559);
			this.splitter2.TabIndex = 2;
			this.splitter2.TabStop = false;
			// 
			// tvFilePlan
			// 
			this.tvFilePlan.ContextMenu = this.cntxtMnuTreeView;
			this.tvFilePlan.Dock = System.Windows.Forms.DockStyle.Left;
			this.tvFilePlan.ImageList = this.imageList1;
			this.tvFilePlan.Location = new System.Drawing.Point(3, 0);
			this.tvFilePlan.Name = "tvFilePlan";
			this.tvFilePlan.Size = new System.Drawing.Size(192, 559);
			this.tvFilePlan.TabIndex = 1;
			this.tvFilePlan.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.tvFilePlan_AfterSelect);
			this.tvFilePlan.BeforeExpand += new System.Windows.Forms.TreeViewCancelEventHandler(this.tvFilePlan_BeforeExpand);
			// 
			// splitter1
			// 
			this.splitter1.Location = new System.Drawing.Point(0, 0);
			this.splitter1.Name = "splitter1";
			this.splitter1.Size = new System.Drawing.Size(3, 559);
			this.splitter1.TabIndex = 0;
			this.splitter1.TabStop = false;
			// 
			// tabLifeCycleReports
			// 
			this.tabLifeCycleReports.Controls.Add(this.lvLifeCycleReports);
			this.tabLifeCycleReports.Location = new System.Drawing.Point(4, 22);
			this.tabLifeCycleReports.Name = "tabLifeCycleReports";
			this.tabLifeCycleReports.Size = new System.Drawing.Size(784, 559);
			this.tabLifeCycleReports.TabIndex = 2;
			this.tabLifeCycleReports.Text = "Life Cycle Reports";
			// 
			// lvLifeCycleReports
			// 
			this.lvLifeCycleReports.ContextMenu = this.cntxtMnuBatchList;
			this.lvLifeCycleReports.Dock = System.Windows.Forms.DockStyle.Fill;
			this.lvLifeCycleReports.FullRowSelect = true;
			this.lvLifeCycleReports.Location = new System.Drawing.Point(0, 0);
			this.lvLifeCycleReports.Name = "lvLifeCycleReports";
			this.lvLifeCycleReports.Size = new System.Drawing.Size(784, 559);
			this.lvLifeCycleReports.TabIndex = 0;
			this.lvLifeCycleReports.View = System.Windows.Forms.View.Details;
			// 
			// cntxtMnuBatchList
			// 
			this.cntxtMnuBatchList.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																							  this.mnuViewLifeCycleResults,
																							  this.mnuTransitionLifeCycleResults,
																							  this.menuViewTransitionResults,
																							  this.mnuDeleteLifeCycleResults});
			// 
			// mnuViewLifeCycleResults
			// 
			this.mnuViewLifeCycleResults.Index = 0;
			this.mnuViewLifeCycleResults.Text = "View Results";
			this.mnuViewLifeCycleResults.Click += new System.EventHandler(this.mnuViewLifeCycleResults_Click);
			// 
			// mnuTransitionLifeCycleResults
			// 
			this.mnuTransitionLifeCycleResults.Index = 1;
			this.mnuTransitionLifeCycleResults.Text = "Transition";
			this.mnuTransitionLifeCycleResults.Click += new System.EventHandler(this.mnuTransitionLifeCycleResults_Click);
			// 
			// menuViewTransitionResults
			// 
			this.menuViewTransitionResults.Index = 2;
			this.menuViewTransitionResults.Text = "Show Transition Results...";
			this.menuViewTransitionResults.Click += new System.EventHandler(this.menuViewTransitionResults_Click);
			// 
			// mnuDeleteLifeCycleResults
			// 
			this.mnuDeleteLifeCycleResults.Index = 3;
			this.mnuDeleteLifeCycleResults.Text = "Delete";
			this.mnuDeleteLifeCycleResults.Click += new System.EventHandler(this.mnuDeleteLifeCycleResults_Click);
			// 
			// tabTextPage
			// 
			this.tabTextPage.Controls.Add(this.textBox1);
			this.tabTextPage.Location = new System.Drawing.Point(4, 22);
			this.tabTextPage.Name = "tabTextPage";
			this.tabTextPage.Size = new System.Drawing.Size(784, 559);
			this.tabTextPage.TabIndex = 1;
			this.tabTextPage.Text = "Log Messages";
			// 
			// textBox1
			// 
			this.textBox1.Dock = System.Windows.Forms.DockStyle.Fill;
			this.textBox1.Location = new System.Drawing.Point(0, 0);
			this.textBox1.Multiline = true;
			this.textBox1.Name = "textBox1";
			this.textBox1.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.textBox1.Size = new System.Drawing.Size(784, 559);
			this.textBox1.TabIndex = 0;
			this.textBox1.Text = "";
			// 
			// statusBar1
			// 
			this.statusBar1.Location = new System.Drawing.Point(0, 563);
			this.statusBar1.Name = "statusBar1";
			this.statusBar1.ShowPanels = true;
			this.statusBar1.Size = new System.Drawing.Size(792, 22);
			this.statusBar1.TabIndex = 9;
			this.statusBar1.Text = "statusBar1";
			// 
			// menuItem7
			// 
			this.menuItem7.Index = 6;
			this.menuItem7.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuTemplNoneTreeItem,
																					  this.mnuTemplExpandLstTreeItem,
																					  this.mnuTemplProfilesTreeItem,
																					  this.mnuTemplListsProfilesTreeItem});
			this.menuItem7.Text = "Template XML";
			// 
			// mnuTemplNoneTreeItem
			// 
			this.mnuTemplNoneTreeItem.Index = 0;
			this.mnuTemplNoneTreeItem.Text = "NONE";
			this.mnuTemplNoneTreeItem.Click += new System.EventHandler(this.mnuTemplNoneTreeItem_Click);
			// 
			// mnuTemplExpandLstTreeItem
			// 
			this.mnuTemplExpandLstTreeItem.Index = 1;
			this.mnuTemplExpandLstTreeItem.Text = "ExpandLists";
			this.mnuTemplExpandLstTreeItem.Click += new System.EventHandler(this.mnuTemplExpandLstTreeItem_Click);
			// 
			// mnuTemplProfilesTreeItem
			// 
			this.mnuTemplProfilesTreeItem.Index = 2;
			this.mnuTemplProfilesTreeItem.Text = "Profiles";
			this.mnuTemplProfilesTreeItem.Click += new System.EventHandler(this.mnuTemplProfilesTreeItem_Click);
			// 
			// mnuTemplListsProfilesTreeItem
			// 
			this.mnuTemplListsProfilesTreeItem.Index = 3;
			this.mnuTemplListsProfilesTreeItem.Text = "ListsAndProfiles";
			this.mnuTemplListsProfilesTreeItem.Click += new System.EventHandler(this.mnuTemplListsProfilesTreeItem_Click);
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(792, 585);
			this.Controls.Add(this.statusBar1);
			this.Controls.Add(this.tabControl2);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu1;
			this.Name = "Form1";
			this.Text = ((string)(configurationAppSettings.GetValue("Form1.Text", typeof(string))));
			this.Load += new System.EventHandler(this.Form1_Load);
			this.tabControl2.ResumeLayout(false);
			this.tabFilePlanExplorer.ResumeLayout(false);
			this.tabLifeCycleReports.ResumeLayout(false);
			this.tabTextPage.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		#region Utility

		/// <summary>
		/// Returns the current ClientSession object.
		/// </summary>
		public ClientSession irmFactory
		{
			get
			{
				return this.m_session;
			}
		}

		/// <summary>
		/// Activates the File Plan Explorer tab.
		/// </summary>
		public void SetFilePlanTabActive()
		{
			tabControl2.SelectedTab = tabFilePlanExplorer;
		}

		/// <summary>
		/// Given the File Plan Component ID, the View ID and the Definition ID of a component
		/// this method searches the File Plan for it's location.
		/// The method expands all tree nodes along the path to the component
		/// and then highlights the component node.
		/// Given enough info you can automatically find anything.
		/// </summary>
		/// <param name="nFlPnCmpntId">The Id of the component to find.</param>
		/// <param name="viewId">The View to look for the component in.</param>
		/// <param name="fplnCmpntDefId">The Definition Id of the component.</param>
		public void FindTreeNode(long nFlPnCmpntId, int viewId, long fplnCmpntDefId)
		{
			if(viewId == 0)
			{
				if(fplnCmpntDefId != 0)
				{
					string xml = m_session.getDefinitionController().getComponentDefinitionById(fplnCmpntDefId);
					IRMList list = new IRMList(xml,"PrmViewID","FlPlnViewNm");
					viewId = Convert.ToInt32(list.PrimaryKey(0));
				}
			}
			if(viewId == 0)
			{
				return; //Nothing we can do without viewId
			}
			string fullPath = m_session.getNavigationController().getFullPathById(nFlPnCmpntId, viewId, (int)IRMConstants.DualNamingOptions.BY_NAME);
			string sysFullPathPrefix = m_session.getConfigurationController().getEngineConfigValue("sysFullPathPrefix",0);
			string sysFullPathLevelDelimeter = m_session.getConfigurationController().getEngineConfigValue("sysFullPathLevelDelimiter",0);

			//Split the full pathh up into an array of names
			string temp = fullPath.Replace(sysFullPathPrefix,"");
			char [] delimiter = sysFullPathLevelDelimeter.ToCharArray();
			string[] str = temp.Split(delimiter);

			TreeNode node = null;

			//Get view Node first
			string name = str[0];
			TreeNodeCollection views = tvFilePlan.Nodes[0].Nodes;
			for(int i = 0; i < views.Count; i++)
			{
				if(name == views[i].Text)
				{
					node = views[i];
				}
			}
			//Then expand all nodes along the path
			for(int i = 1; i  < str.Length; i++)
			{
				IRMComponent irm = (IRMComponent)node.Tag;
				name = str[i];
				bool newTargets = false;
				int count = node.Nodes.Count;
				//Check if targets exist
				if(count == 1)
				{
					//If the target list is not there add it
					if(node.FirstNode.Tag == null)
					{
						this.Cursor = Cursors.WaitCursor;
						CommonTasks.addTargetList(m_session, node,irm.FlPlnViewID,irm.FlPlnCmpntID,util);
						this.Cursor = Cursors.Default;
						newTargets = true;
					}
				}
				if(!newTargets)//Don't have to refresh if you just added the list
				{
					//Refresh the node list incase new components were added by another client
					this.Cursor = Cursors.WaitCursor;
					CommonTasks.Refresh(m_session,node,irm.FlPlnViewID,irm.FlPlnCmpntID,false,false);
					this.Cursor = Cursors.Default;
				}
				node.Expand();
				count = node.Nodes.Count;
				for(int j = 0; j < count; j++)
				{
					if(node.Nodes[j].Text == name)
					{
						node = node.Nodes[j];
						irm = (IRMComponent)node.Tag; 
						//If this is the last node then we can display the targets for it
						if(i == str.Length - 1)
						{
							node.EnsureVisible();
							tvFilePlan.SelectedNode = node;
							this.Cursor = Cursors.WaitCursor;
							CommonTasks.DisplayRunListing(m_session,lvComponents,irm.FlPlnViewID,irm.FlPlnCmpntID);
							this.Cursor = Cursors.Default;
						}
						break;
					}
				}
			}
		}

		/// <summary>
		/// This method appends text to the text box on the
		/// Log Messages Tab. 
		/// </summary>
		/// <param name="location">The module that is the source of the message.</param>
		/// <param name="msg">Some message.</param>
		public void log(string location, string msg)
		{
			textBox1.AppendText("\r\n" + location + "\r\n\t" + msg);
		}

		/// <summary>
		/// This method appends text to the text box on the
		/// Log Messages Tab.
		/// </summary>
		/// <param name="msg">Some message.</param>
		public void appendText(string msg)
		{
			textBox1.AppendText(msg);
		}

		/// <summary>
		/// This method creates a new Client Session object (if needed)
		/// using information determined in the Form1 constructor
		/// and then calls the appropriate login method.
		/// </summary>
		/// <param name="isHostLogin">If true, specifies that the hostLogin method should
		/// be called.</param>
		/// <returns>true if the login was successfull</returns>
		private bool initClientSession(bool isHostLogin)
		{
			bool bFound = false;
			this.m_databaseJNDIName = null;
			
			if(!m_bLoggedIn)
			{
				m_session = new ClientSession(this.m_serverNamePort);
			}
				
			//Get the XML containing the list of available data sources
			string dataSourceList = m_session.getDataSourceList();

			//util.log(dataSourceList);

			//Get the JNDIName name for database
			IRMList list = new IRMList(dataSourceList,"JNDIName","Name");

			if(list.Count > 0)
			{
				for(int i=0; i < list.Count; i++)
				{
					string name = list.Caption(i);
					if(name.Equals(this.m_databaseName))
					{
						this.m_databaseJNDIName = list.PrimaryKey(i);
						bFound = true;
						break;
					}
				}
				if(!bFound)
				{
					string temp = this.m_databaseName;
					this.m_databaseName = list.Caption(0);
					this.m_databaseJNDIName = list.PrimaryKey(0);

					MessageBox.Show("The Database: " + temp + " was not found in the list of available databases. " +
						"Using the first available database: " + this.m_databaseName);
				}
				if(isHostLogin)
				{
					m_session.hostLogin(this.m_extUserId,"QAHost","administrator","cronos",this.m_databaseJNDIName,"",this.m_language);
					statusBar1.Text = "External user: " + this.m_extUserId + "\r\nServer : " + this.m_serverNamePort + "\r\ndatabase JNDIName: " + this.m_databaseJNDIName;
					m_bLoggedIn = true;
					return true;
				}
				else
				{
					LoginForm frm = new LoginForm(m_session, this, list, this.m_userName, this.m_databaseName, this.m_language);
					if(frm.ShowDialog(this) == DialogResult.OK)
					{
						m_bLoggedIn = true;
						statusBar1.Text = "Internal User: " + frm.SelectedUserName + "\r\nServer : " + this.m_serverNamePort + "\r\ndatabase JNDIName: " + frm.SelectedDatabaseName;
						return true;
					}
					else
					{
						return false;
					}
				}
			}
			else
			{
				util.log("No database information");
				return false;
			}
		}

		#endregion

		#region treeViewUtilityMethods

		/// <summary>
		/// Delete the specified File Plan Component and all it's children.
		/// </summary>
		/// <param name="nFlPlnCmpntID">The Id of the component to delete.</param>
		/// <param name="strFlPlnCmpntNm">The name (for display purposes) of the component to delete.</param>
		/// <returns></returns>
		private bool DeleteFilePlanComponent(long nFlPlnCmpntID, string strFlPlnCmpntNm)
		{
			string msg = "Delete Component: " + strFlPlnCmpntNm;
			if(MessageBox.Show(this,msg,"Component Operations",MessageBoxButtons.YesNo) == DialogResult.Yes)
			{
				try
				{
					m_session.getFilePlanComponentController().deleteFilePlanComponent(nFlPlnCmpntID,1);
					return true;
				}
				catch(System.Net.WebException we)
				{
					util.logWebException(we);
				}
				catch(System.Web.Services.Protocols.SoapException s)
				{
					util.handleSoapException(s,false); //false indicates we will not check for time ou here
					util.showSoapException(s);
				}
			}
			return false;
		}

		/// <summary>
		/// Delete the specified File Plan View.
		/// Note it will not be possible to delete a View unless
		/// all it's children and relationship definitions have been deleted.
		/// </summary>
		/// <param name="nFlPlnViewID">The Id of the View to delete.</param>
		/// <param name="strFlPlnViewNm">The name (for display purposes only) to delete.</param>
		/// <returns></returns>
		private bool DeleteFilePlanView(int nFlPlnViewID,string strFlPlnViewNm)
		{
			string msg = "Delete View: " + strFlPlnViewNm;
			if(MessageBox.Show(this,msg,"Component Operations",MessageBoxButtons.YesNo) == DialogResult.Yes)
			{
				try
				{
					m_session.getFilePlanDesignController().deleteFilePlanView(nFlPlnViewID);
					return true;
				}
				catch(System.Net.WebException we)
				{
					util.logWebException(we);
				}
				catch(System.Web.Services.Protocols.SoapException s)
				{
					util.handleSoapException(s,false); //false indicates we will not check for time ou here
					util.showSoapException(s);
				}
			}
			return false;
		}

		/// <summary>
		/// Automatically generates child components under the selected component node.
		/// </summary>
		/// <param name="irm">An IRMComponent containing information for the selected node.</param>
		private void autoGen(IRMComponent irm)
		{
			addFilePlanComponents(irm.FlPlnCmpntID, irm.FlPlnViewID, 10);
		}

		#endregion

		#region treeViewEvents
		/// <summary>
		/// This event is called after a tree node is selected.
		/// When a tree node is selected the children
		/// for the selected node will be displayed in the corresponding List View.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tvFilePlan_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag;
			m_lastSelectedNode = tvFilePlan.SelectedNode;

			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT || irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				this.Cursor = Cursors.WaitCursor;
				CommonTasks.DisplayRunListing(m_session, lvComponents, irm.FlPlnViewID,irm.FlPlnCmpntID);
				this.Cursor = Cursors.Default;
			}
			else if(irm.FlPlnCmpntDefID == IRMComponent.ROOT)
			{
				DisplayViewList(false);
			}
			else
			{
				lvComponents.Items.Clear();
			}
		}

		/// <summary>
		/// This event responds to the tree view context menu delete command.
		/// The delete operation depends upon the type of component selected.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuDeleteTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag;
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
			{
				this.Cursor = Cursors.WaitCursor;
				if(DeleteFilePlanComponent(irm.FlPlnCmpntID, irm.FlPlnCmpntNm))
				{
					tvFilePlan.SelectedNode.Remove();
				}
				this.Cursor = Cursors.Default;
			}
			else if(irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				this.Cursor = Cursors.WaitCursor;
				if(DeleteFilePlanView(irm.FlPlnViewID,irm.FlPlnViewNm))
				{
					tvFilePlan.SelectedNode.Remove();
				}
				this.Cursor = Cursors.Default;
			}
			lvComponents.Items.Clear();
			lvComponents.Columns.Clear();
		}

		/// <summary>
		/// This event responds to the tree view Life Cycle Query... command.
		/// First a Saved Life Cycle Query list will be displayed then the Query will
		/// be executed using the Full Path of the selected node as the context
		/// for the search.
		/// The Search results are displayed in a modeless window.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void menuLifeCycleQuery_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			m_lastSelectedNode = tvFilePlan.SelectedNode;

			string xmlQuery; // = m_session.getQueryController().getQueryById(101);

			SavedSearchesForm srchForm = new SavedSearchesForm(m_session, (byte)IRMConstants.QueryTypes.LIFE_CYCLE);
			if(srchForm.ShowDialog(this) == DialogResult.OK)
			{
				xmlQuery = srchForm.XMLQuery;	
			}
			else
			{
				return;
			}
			string strScope = "";
			if(irm.FlPlnViewID != 0)
			{
				strScope = m_session.getNavigationController().getFullPathById(irm.FlPlnCmpntID,(long)irm.FlPlnViewID,(int)IRMConstants.DualNamingOptions.BY_NAME);
			}
			
			IRMBatchQuery irmQuery = new IRMBatchQuery(xmlQuery, m_session, strScope, true, OptionalFields.REFERENCE_DATE_TO);
			irmQuery.Initialize();

			//TODO no such thing as IRMConstants.QueryResultOptions.NORMAL for Batch Ops???
			QueryForm frm = new QueryForm(this, irmQuery,"Fill in all the parameter values to complete the following Life Cycle Query.",(int)IRMConstants.QueryResultOptions.INCLUDE_COLUMNS);
			if(frm.ShowDialog(this) == DialogResult.OK)
			{
				IRMTaskResultsForm frmTaskResultsForm = new IRMTaskResultsForm(frm.getIRMBatchQuery());
				if(frmTaskResultsForm.ShowDialog(this) == DialogResult.OK)
				{
					IRMLifeCycleResultList rl = frmTaskResultsForm.LifeCycleResultList;

					//TODO improve this global
					string strTaskId = frmTaskResultsForm.TaskID;

					//Make sure that only the following columns will be displayed
					rl.AddColumn("NameFullPath");
					rl.AddColumn("Conflict");
					rl.AddColumn("NextEligiblePhaseName");
					rl.AddColumn("NextEligiblePhaseDate");

					LifeCycleResultListForm lcrlFrm = new LifeCycleResultListForm(strTaskId, m_session,this,rl,irm.FlPlnViewID,true);
					lcrlFrm.Show();
				}
			}
		}

		/// <summary>
		/// This event responds to the tree view Life Cycle Query... command.
		/// First a Saved Query list will be displayed then the Query will
		/// be executed using the Full Path of the selected node as the context
		/// for the search.
		/// The Search results are displayed in a modeless window.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuSearchTreeItem_Click(object sender, System.EventArgs e)
		{
			
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			m_lastSelectedNode = tvFilePlan.SelectedNode;

			string xmlQuery; // = m_session.getQueryController().getQueryById(101);

			SavedSearchesForm srchForm = new SavedSearchesForm(m_session, (byte)IRMConstants.QueryTypes.ADVANCED);
			if(srchForm.ShowDialog(this) == DialogResult.OK)
			{
				xmlQuery = srchForm.XMLQuery;	
			}
			else
			{
				return;
			}
			string strScope = "";
			if(irm.FlPlnViewID != 0)
			{
				strScope = m_session.getNavigationController().getFullPathById(irm.FlPlnCmpntID,(long)irm.FlPlnViewID,(int)IRMConstants.DualNamingOptions.BY_NAME);
			}
			
			//Query constructor
			//Note the addDisplayInfo flag must be set to true if we are going to use
			//Display information to create a Query Form
			IRMQuery irmQuery = new IRMQuery(xmlQuery, m_session, strScope, true);
			irmQuery.Initialize(); //Note this method must always be called first.
			QueryForm frm = new QueryForm(this, irmQuery, "Fill in all the parameter values to complete the following Query.",(int)IRMConstants.QueryResultOptions.INCLUDE_COLUMNS);
			if(frm.ShowDialog(this) == DialogResult.OK)
			{
				IRMResultList rl = new IRMResultList(irmQuery);
				//DisplayResultList(rl, irm.FlPlnViewID);
				ResultListForm rlFrm = new ResultListForm(m_session, this, rl, irm.FlPlnViewID);
				rlFrm.Show();
			}
		}

		/// <summary>
		/// This event responds to the tree view Add command.
		/// If the selected component has more than one type of
		/// target that is allowed to be added then a list window is displayed
		/// showing available target types.
		/// If only one type is allowed then the component properties
		/// form is displayed immediately.
		/// Note the Component Properties form that is displayed uses the current
		/// users assigned profile and default values.
		/// Currently this form only displays the most primitive type of imput
		/// controls (Text Boxes) and no signicant client side business logic
		/// or validation is performed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuAddTreeItemTarget_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag;
			m_lastSelectedNode = tvFilePlan.SelectedNode;

			if(irm.FlPlnCmpntDefID == IRMComponent.ROOT)
			{
				string xml = m_session.getTemplateController().getNewComponent((long)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW,0,0,(int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);
				util.log(xml);
				IRMFields fields = new IRMFields(xml);

				ComponentForm cmpntFrm = new ComponentForm(m_session,irm,this,fields,ComponentForm.ADD);
				if(cmpntFrm.ShowDialog(this) == DialogResult.OK)
				{
					this.Cursor = Cursors.WaitCursor;
					CommonTasks.DisplayRunListing(m_session, lvComponents, irm.FlPlnViewID,irm.FlPlnCmpntID);
					CommonTasks.Refresh(m_session, tvFilePlan.SelectedNode,irm.FlPlnViewID,irm.FlPlnCmpntID,false,false);
					this.Cursor = Cursors.Default;
				}

				//MessageBox.Show("Add View - Code under construction");
			}
			if(irm.FlPlnCmpntDefID > 0 || irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				//AddTarget(irm);
				string xmlList = m_session.getFilePlanComponentController().getAllowedTargetList(irm.FlPlnCmpntID,irm.FlPlnViewID);
				IRMList list = new IRMList(xmlList,"FlPlnCmpntDefTgtID","FlPlnCmpntDefNm");
				
				for(int i = list.Count - 1; i > -1; i--)
				{
					int nPrmViewID = Convert.ToInt32(list.getAttributeValue(i,"PrmViewID"));
					if(nPrmViewID != irm.FlPlnViewID)
					{
						list.removeItem(i);
					}
				}

				if(list.Count > 0)
				{
					long nFlPlnCmpntDefTgtID = 0;
					if(list.Count > 1)
					{
						AllowedTargetsForm frm = new AllowedTargetsForm(list);
						if(frm.ShowDialog(this) == DialogResult.OK)
						{
							nFlPlnCmpntDefTgtID = frm.FlPlnCmpntDefTgtID;
						}
					}
					else
					{
						nFlPlnCmpntDefTgtID = Convert.ToInt32(list.PrimaryKey(0));
					}
					//MessageBox.Show("FlPlnCmpntDefTgtID = " + nFlPlnCmpntDefTgtID);

					string xml = m_session.getTemplateController().getNewComponent(nFlPlnCmpntDefTgtID,0,0,(int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);
					IRMFields fields = new IRMFields(xml);
					fields.hideSecurityFields(); //This is a convenience method to hide security fields if they are not being used
					fields.inactivateSecurityFields(); //This is a convenience method to make sure security fields are not added
					
					//Hide some fields we never want to display
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("FlPlnCmpntID",false);
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("LastAddtnDt",false);
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("LastRtrvDt",false);

					//TODO Vital Records Info Under Construction
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("VitalRecordConfig",false);
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("VitalRecordConfig",false);

					
					//Inactivate fields we do not want to be set in the Fields Collection
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("FlPlnCmpntID",false);
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("LastAddtnDt",false);
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("LastRtrvDt",false);
					

					ComponentForm cmpntFrm = new ComponentForm(m_session,irm,this,fields,ComponentForm.ADD);
					if(cmpntFrm.ShowDialog(this) == DialogResult.OK)
					{
						this.Cursor = Cursors.WaitCursor;
						CommonTasks.DisplayRunListing(m_session, lvComponents, irm.FlPlnViewID,irm.FlPlnCmpntID);
						CommonTasks.Refresh(m_session, tvFilePlan.SelectedNode,irm.FlPlnViewID,irm.FlPlnCmpntID,false,false);
						this.Cursor = Cursors.Default;
					}
				}
				else
				{
					MessageBox.Show("There are no Add Relationships defined for this Source");
				}
			}
		}

		/// <summary>
		/// This event responds to the tree view Edit command.
		/// Note the Component Properties form that is displayed uses the current
		/// users assigned profile and current values.
		/// Currently this form only displays the most primitive type of imput
		/// controls (Text Boxes) and no signicant client side business logic
		/// or validation is performed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuEditTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag;
			m_lastSelectedNode = tvFilePlan.SelectedNode;

			if(irm.FlPlnCmpntDefID == IRMComponent.ROOT)
			{
				MessageBox.Show("Add View - Code under construction");
			}
			if(irm.FlPlnCmpntDefID > 0 || irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				string xml = m_session.getTemplateController().getFilePlanComponent(irm.FlPlnCmpntID,0,(int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);

				IRMFields fields = new IRMFields(xml);
				fields.hideSecurityFields(); //This is a convenience method to hide security fields if they are not being used
				fields.inactivateSecurityFields(); //This is a convenience method to make sure security fields are not added
					
				//Hide some fields we never want to display
				fields.setVisabilityByFlPlnCmpntAttrDefColNm("FlPlnCmpntID",false);
				fields.setVisabilityByFlPlnCmpntAttrDefColNm("LastAddtnDt",false);
				fields.setVisabilityByFlPlnCmpntAttrDefColNm("LastRtrvDt",false);
					
				//Inactivate fields we do not want to be set in the Fields Collection
				fields.setActiveStateByFlPlnCmpntAttrDefColNm("FlPlnCmpntID",false);
				fields.setActiveStateByFlPlnCmpntAttrDefColNm("LastAddtnDt",false);
				fields.setActiveStateByFlPlnCmpntAttrDefColNm("LastRtrvDt",false);
					
				ComponentForm cmpntFrm = new ComponentForm(m_session, irm, this, fields, ComponentForm.EDIT);
				if(cmpntFrm.ShowDialog(this) == DialogResult.OK)
				{
					//Get the parent node
					TreeNode parent = m_lastSelectedNode.Parent;
					
					//Remove the updated node
					m_lastSelectedNode.Remove();

					//Refresh the local branch of the tree view
					irm = (IRMComponent)parent.Tag;
					this.Cursor = Cursors.WaitCursor;
					CommonTasks.Refresh(m_session,parent,irm.FlPlnViewID,irm.FlPlnCmpntID,false,false);
					this.Cursor = Cursors.Default;
				}
			}
		}

		/// <summary>
		/// Responds to the tree view Autogen command
		/// Automatically generates 10 of each allowed target for the selected component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuAutoGen_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT || irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				this.Cursor = Cursors.WaitCursor;
				autoGen(irm);
				CommonTasks.DisplayRunListing(m_session, lvComponents, irm.FlPlnViewID,irm.FlPlnCmpntID);
				CommonTasks.Refresh(m_session, tvFilePlan.SelectedNode,irm.FlPlnViewID,irm.FlPlnCmpntID,false,false);
				this.Cursor = Cursors.Default;
			}
		}

		private void mnuMetadataTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
			{
				string xml = m_session.getFilePlanComponentController().getFilePlanComponentById(irm.FlPlnCmpntID);
				CommonTasks.DisplayXML(xml);
			}
			if(irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				string xml = m_session.getFilePlanDesignController().getFilePlanViewByID(irm.FlPlnViewID);
				CommonTasks.DisplayXML(xml);
			}
		}

		private void getTemplateData(int options)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
			{
				string xml = m_session.getTemplateController().getFilePlanComponent
					(irm.FlPlnCmpntID, 0, options);
				CommonTasks.DisplayXML(xml);
			}
		}

		private void mnuMetadaPlus_Click(object sender, System.EventArgs e)
		{
			getTemplateData((int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);
		}

		private void mnuTemplNoneTreeItem_Click(object sender, System.EventArgs e)
		{
			getTemplateData((int)IRMConstants.TemplateOptions.NONE);
		}

		private void mnuTemplExpandLstTreeItem_Click(object sender, System.EventArgs e)
		{
			getTemplateData((int)IRMConstants.TemplateOptions.EXPAND_LISTS);
		}

		private void mnuTemplProfilesTreeItem_Click(object sender, System.EventArgs e)
		{
			getTemplateData((int)IRMConstants.TemplateOptions.PROFILE_FIELDS);
		}

		private void mnuTemplListsProfilesTreeItem_Click(object sender, System.EventArgs e)
		{
			getTemplateData((int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);
		}

		private void mnuUserPermissionsTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
			{
				string xml = m_session.getFilePlanComponentController().getEffectiveUserPermissionsList(irm.FlPlnCmpntID);
				CommonTasks.DisplayXML(xml);
			}
		}

		private void mnuExtendedPermissionsTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
			{
				string xml = m_session.getSecurityController().getExtendedPermissionList(irm.FlPlnCmpntID,"",0,0);
				CommonTasks.DisplayXML(xml);
			}
		}

		/// <summary>
		/// Called before the tree view context menu pops up.
		/// Use this method to configure the context menu
		/// depending on the definition type of the selected component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void cntxtMnuTreeView_Popup(object sender, System.EventArgs e)
		{
			mnuAddTreeItemTarget.Enabled = true;
			mnuEditTreeItem.Enabled = true;
			mnuViewTreeItem.Enabled = true;
			mnuDeleteTreeItem.Enabled = true;
			mnuRefreshTreeItem.Enabled = true;
			mnuSearchTreeItem.Enabled = true;
			menuLifeCycleQuery.Enabled = true;
			mnuAutoGen.Enabled = true;
			mnuMetadataTreeItem.Enabled = true;
			mnuUserPermissionsTreeItem.Enabled = true;
			mnuExtendedPermissionsTreeItem.Enabled = true;

			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag; 
			if(irm.FlPlnCmpntDefID > (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_COMPONENT)
			{
			}
			if(irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
			{
				mnuUserPermissionsTreeItem.Enabled = false;
				mnuExtendedPermissionsTreeItem.Enabled = false;
			}
			else if(irm.FlPlnCmpntDefID == IRMComponent.ROOT)
			{
				mnuEditTreeItem.Enabled = false;
				mnuViewTreeItem.Enabled = false;
				mnuDeleteTreeItem.Enabled = false;
				mnuAutoGen.Enabled = false;
				mnuMetadataTreeItem.Enabled = false;
				mnuUserPermissionsTreeItem.Enabled = false;
				mnuExtendedPermissionsTreeItem.Enabled = false;
			}
		}

		/// <summary>
		/// Called just before a tree view item is about to expand.
		/// Use this method to get the list of targets (children) for the
		/// selected component so that they may be displayed.
		/// The event is not triggered if the selected node has already
		/// had this event called upon it.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tvFilePlan_BeforeExpand(object sender, System.Windows.Forms.TreeViewCancelEventArgs e)
		{
			int count = e.Node.Nodes.Count;
			if(count == 1)
			{
				if(e.Node.FirstNode.Tag == null)
				{
					IRMComponent irm = (IRMComponent)e.Node.Tag;
					this.Cursor = Cursors.WaitCursor;
					CommonTasks.addTargetList(m_session, e.Node,irm.FlPlnViewID,irm.FlPlnCmpntID,util);
					this.Cursor = Cursors.Default;
				}
			}
		}

		/// <summary>
		/// Responds to the treeview Refresh command.
		/// This method will walk down through the branches of
		/// the selected node and call the Records Manager Engine
		/// to get updated File Plan information and adjust the
		/// tree view to reflect the current state. 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuRefreshTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag;
			if(irm.FlPlnViewID == 0)
			{
				this.Cursor = Cursors.WaitCursor;
				CommonTasks.RefreshViews(m_session,tvFilePlan.SelectedNode,true);
				this.Cursor = Cursors.Default;

			}
			else
			{
				this.Cursor = Cursors.WaitCursor;
				CommonTasks.Refresh(m_session,tvFilePlan.SelectedNode,irm.FlPlnViewID,irm.FlPlnCmpntID,true,false);
				this.Cursor = Cursors.Default;
			}
		}

		/// <summary>
		/// This event responds to the tree view "View" command.
		/// Note the Component Properties form that is displayed uses the current
		/// users assigned profile and current values.
		/// Currently this form only displays the most primitive type of display
		/// controls (Text Boxes) and no signicant client side business logic
		/// is performed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuViewTreeItem_Click(object sender, System.EventArgs e)
		{
			IRMComponent irm = (IRMComponent)tvFilePlan.SelectedNode.Tag;
			CommonTasks.viewSeletedItem(this,m_session,irm);
		}

		#endregion

		#region ListViewUtilityMethods

		/// <summary>
		/// Gets the List of File Plan Views and displays their information in the
		/// File Plan Explorer List View.
		/// </summary>
		/// <param name="onlyRootNavigation">When true specifies that only views that are "Root" navigable
		/// will be displayed. Only Hierarchical views that contain relationship definitions of the
		/// form Root to Component will be displayed.</param>
		private void DisplayViewList(bool onlyRootNavigation)
		{
			string xml = m_session.getFilePlanDesignController().getFilePlanViewList(onlyRootNavigation);
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.LoadXml(xml);
				XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
				
				lvComponents.Items.Clear();
				lvComponents.Columns.Clear();

				lvComponents.Columns.Add(new ColumnHeader());
				lvComponents.Columns[0].Text = "Component";
				lvComponents.Columns[0].Width = 200;

				lvComponents.Columns.Add(new ColumnHeader());
				lvComponents.Columns[1].Text = "Name";
				lvComponents.Columns[1].Width = 200;
				
				for(int i = 0; i < itemNodes.Count; i++)
				{
					int nFlPlnViewID = System.Convert.ToInt32(itemNodes.Item(i).Attributes.GetNamedItem("FlPlnViewID").Value);
					string strFlPlnViewNm = itemNodes.Item(i).Attributes.GetNamedItem("FlPlnViewNm").Value;
					IRMComponent irm = new IRMComponent(nFlPlnViewID,strFlPlnViewNm);
					ListViewItem lvItem = lvComponents.Items.Add(new ListViewItem(
						new string[]{irm.FlPlnCmpntDefNm,irm.FlPlnViewNm,""},1 ));
					lvItem.Tag = irm;
				}
			}
			catch(XmlException ed)
			{
				util.log("LoadXml Error: " + ed.Message);
			}
		}

		/// <summary>
		/// Refreshes the tree branch under the specified
		/// View. Used when components are put in a container (linked to an external source).
		/// </summary>
		/// <param name="containerViewId">The view containig the source that is being linked to.</param>
		private void RefreshContainerView(int containerViewId)
		{
			//Look through existing View Nodes for the containerViewId
			TreeNodeCollection viewNodes = tvFilePlan.Nodes[0].Nodes;
			for(int i = 0; i < viewNodes.Count; i++)
			{
				IRMComponent irm = (IRMComponent)viewNodes[i].Tag;
				if(irm.FlPlnViewID == containerViewId)
				{
					this.Cursor = Cursors.WaitCursor;
					CommonTasks.Refresh(m_session,viewNodes[i],containerViewId,0,true,false);
					this.Cursor = Cursors.Default;
				}
			}
		}

		#endregion

		#region ListViewEvents

		/// <summary>
		/// This event responds to the List view Edit command.
		/// Note the Component Properties form that is displayed uses the current
		/// users assigned profile and default values.
		/// Currently this form only displays the most primitive type of imput
		/// controls (Text Boxes) and no signicant client side business logic
		/// or validation is performed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuEditListItem_Click(object sender, System.EventArgs e)
		{
			int count = lvComponents.SelectedItems.Count;
			if(count > 0)
			{
				IEnumerator items = lvComponents.SelectedItems.GetEnumerator();
				items.MoveNext();
				ListViewItem lvi = (ListViewItem)items.Current;
				IRMComponent irm = (IRMComponent)lvi.Tag;
				if(irm.FlPlnCmpntDefID > 0 || irm.FlPlnCmpntDefID == (int)IRMConstants.ComponentDefinitionTypes.FILE_PLAN_VIEW)
				{
					string xml = m_session.getTemplateController().getFilePlanComponent(irm.FlPlnCmpntID,0,(int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);

					IRMFields fields = new IRMFields(xml);
					fields.hideSecurityFields(); //This is a convenience method to hide security fields if they are not being used
					fields.inactivateSecurityFields(); //This is a convenience method to make sure security fields are not added
						
					//Hide some fields we never want to display
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("FlPlnCmpntID",false);
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("LastAddtnDt",false);
					fields.setVisabilityByFlPlnCmpntAttrDefColNm("LastRtrvDt",false);
						
					//Inactivate fields we do not want to be set in the Fields Collection
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("FlPlnCmpntID",false);
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("LastAddtnDt",false);
					fields.setActiveStateByFlPlnCmpntAttrDefColNm("LastRtrvDt",false);
						
					ComponentForm cmpntFrm = new ComponentForm(m_session, irm ,this,fields,ComponentForm.EDIT);
					if(cmpntFrm.ShowDialog(this) == DialogResult.OK)
					{
						//Refresh the List View
						irm = (IRMComponent)m_lastSelectedNode.Tag;
						this.Cursor = Cursors.WaitCursor;
						CommonTasks.DisplayRunListing(m_session, lvComponents, irm.FlPlnViewID,irm.FlPlnCmpntID);
						this.Cursor = Cursors.Default;
					}
				}
			}
		}

		/// <summary>
		/// Responds to the List View context menu "Put in Container" command.
		/// This method takes the selected components and adds a specified
		/// source to each component. This is sometimes called migration.
		/// The selected components (if allowed) will become targets to
		/// the specified source component.
		/// This can simulate physical records being placed in a box.
		/// The records are physically in the box yet they still belong
		/// to a logical classification scheme.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuMigrateListItems_Click(object sender, System.EventArgs e)
		{
			int count = lvComponents.SelectedItems.Count;
			if(count > 0)
			{
				if(m_LookUp == null)
				{
					m_LookUp = new LookUpForm(m_session,this);
				}
				if(m_LookUp.ShowDialog(this) == DialogResult.OK)
				{
					string msg = "Put the Selected Items in the Container: " + m_LookUp.SelectedComponent.FlPlnCmpntNm;
					if(MessageBox.Show(this,msg,"Component Operations",MessageBoxButtons.YesNo) == DialogResult.No)
					{
						return;
					}
					IEnumerator items = lvComponents.SelectedItems.GetEnumerator();
					IRMComponent irm;
					while(items.MoveNext())
					{
						ListViewItem lvi = (ListViewItem)items.Current;
						irm = (IRMComponent)lvi.Tag;
						string sourcList =
						"<List>" +
							"<Item FlPlnCmpntID=\"" + m_LookUp.SelectedComponent.FlPlnCmpntID +
								"\" FlPlnViewID=\"" + m_LookUp.SelectedComponent.FlPlnViewID + "\" />" +
						"</List>"; 
						try
						{
							m_session.getFilePlanComponentController().addSource(irm.FlPlnCmpntID,sourcList);
							lvi.Selected = false;
						}
						catch(System.Net.WebException we)
						{
							util.logWebException(we);
							return;
						}
						catch(System.Web.Services.Protocols.SoapException s)
						{
							util.handleSoapException(s,false); //false indicates we will not check for time out here
							util.showSoapException(s);
							return;
						}
					}
					
					RefreshContainerView(m_LookUp.SelectedComponent.FlPlnViewID);

					//TODO how do we refresh the container node
					//Note m_LookUp.SelectedNode only refreshes lookup
					//this.Cursor = Cursors.WaitCursor;
					//CommonTasks.Refresh(m_session,m_LookUp.SelectedNode,m_LookUp.SelectedComponent.FlPlnViewID,m_LookUp.SelectedComponent.FlPlnCmpntID,true,false);
					//this.Cursor = Cursors.Default;
				}
			}
		}

		/// <summary>
		/// Responds to the List View context menu "Remove from Container" command.
		/// This method takes the selected components and removes them from
		/// the source that they are displayed in.
		/// This could simulate removing records from a box.
		/// Note components cannot remove the source link in their primary view.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuUnMigrateListItems_Click(object sender, System.EventArgs e)
		{
			int count = lvComponents.SelectedItems.Count;
			if(count > 0)
			{
				IEnumerator items = lvComponents.SelectedItems.GetEnumerator();
				IRMComponent irm;
				irm = (IRMComponent)m_lastSelectedNode.Tag;
				long sourceId = irm.FlPlnCmpntID;
				int viewId = irm.FlPlnViewID;
				string msg = "Remove the Selected Items from the Container: " + irm.FlPlnCmpntNm;
				if(MessageBox.Show(this,msg,"Component Operations",MessageBoxButtons.YesNo) == DialogResult.No)
				{
					return;
				}
				items.Reset();
				while(items.MoveNext())
				{
					ListViewItem lvi = (ListViewItem)items.Current;
					irm = (IRMComponent)lvi.Tag;
					try
					{
						m_session.getFilePlanComponentController().removeFromSource(irm.FlPlnCmpntID,sourceId,viewId);
						lvi.Remove();
					}
					catch(System.Net.WebException we)
					{
						util.logWebException(we);
						return;
					}
					catch(System.Web.Services.Protocols.SoapException s)
					{
						util.handleSoapException(s,false); //false indicates we will not check for time out here
						util.showSoapException(s);
						return;
					}
				}
				this.Cursor = Cursors.WaitCursor;
				CommonTasks.Refresh(m_session,m_lastSelectedNode,viewId,sourceId,true,true);
				this.Cursor = Cursors.Default;
			}
		}

		/// <summary>
		/// This event responds to the List view "View" command.
		/// Note the Component Properties form that is displayed uses the current
		/// users assigned profile and current values.
		/// Currently this form only displays the most primitive type of display
		/// controls (Text Boxes) and no signicant client side business logic
		/// is performed.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuViewListItem_Click(object sender, System.EventArgs e)
		{
			int count = lvComponents.SelectedItems.Count;
			if(count > 0)
			{
				IEnumerator items = lvComponents.SelectedItems.GetEnumerator();
				items.MoveNext();
				ListViewItem lvi = (ListViewItem)items.Current;
				IRMComponent irm = (IRMComponent)lvi.Tag;
				CommonTasks.viewSeletedItem(this,m_session,irm);
			}
		}

		/// <summary>
		/// This method deletes the selected list of File Plan Components.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuDeleteListItem_Click(object sender, System.EventArgs e)
		{
			this.Cursor = Cursors.WaitCursor;
			CommonTasks.DeleteListItems(this,m_session,util,lvComponents);
			this.Cursor = Cursors.Default;
		}

		/// <summary>
		/// Called before the List View context menu pops up.
		/// Use this method to configure the context menu
		/// depending on the definition type of the selected component.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void cntxtMnuListView_Popup(object sender, System.EventArgs e)
		{
			mnuAddListItemTarget.Enabled = true;
			mnuEditListItem.Enabled = true;
			mnuViewListItem.Enabled = true;
			mnuDeleteListItem.Enabled = true;
			mnuMigrateLitsItems.Enabled = true;
			mnuUnMigrateListItems.Enabled = true;

			int count = lvComponents.SelectedItems.Count;
			if(count > 0)
			{
				IEnumerator items = lvComponents.SelectedItems.GetEnumerator();
				items.MoveNext();
				ListViewItem lvi = (ListViewItem)items.Current;
				IRMComponent irm = (IRMComponent)lvi.Tag; 
				if(irm.FlPlnCmpntID != 0)
				{

				}
				else
				{
					mnuEditListItem.Enabled = false;
					mnuViewListItem.Enabled = false;
					mnuDeleteListItem.Enabled = false;
					mnuMigrateLitsItems.Enabled = false;
					mnuUnMigrateListItems.Enabled = false;
				}
			}
		}
		#endregion

		#region UtilityMethods

		/// <summary>
		/// Loads the Master Description Dom document from
		/// m_apiRefLocation + "\\Resources\\MasterDescriptionFile.xml";
		/// </summary>
		private bool loadMasterDescription()
		{
			if(m_apiRefLocation == null)
			{
				if(!selectAPIRefLocation()) return false;
			}
			
			//C:\Dev\IBM\Slammer\EngineWorkspace\API_Reference\Resources
			
			m_masterDescFile = m_apiRefLocation + "\\Resources\\MasterDescriptionFile.xml";
			if(!File.Exists(m_masterDescFile))  
			{
				String msg = "Can't find file " + m_masterDescFile + ".\r\n";
				msg += " You should choose the folder which contains \"\\Resources\\MasterDescriptionFile.xml\".";  
				MessageBox.Show(msg);
				return false;
			}
			m_masterDescriptionDoc = new XmlDocument();
			m_masterDescriptionDoc.Load(m_masterDescFile);
			m_masterDescriptionRoot = m_masterDescriptionDoc.DocumentElement;
			m_masterDescriptionComponentList = m_masterDescriptionRoot.GetElementsByTagName("Component");
			
			

			return true;
		}

		private void loadXSDAttributeXML()
		{
			if(m_attributeTypeDoc == null)
			{
				m_AttributeTypeFile = m_apiRefLocation + "\\Resources\\AttributeType.xml";
				m_attributeTypeDoc = new XmlDocument();
				m_attributeTypeDoc.Load(m_AttributeTypeFile);
				XmlElement root = m_attributeTypeDoc.DocumentElement;
				m_attributeTypeItemList = root.GetElementsByTagName("Item");
			}
		}

		/// <summary>
		/// This method will read in an AttributeType.xml file
		/// and insert the records manager data type for found items.
		/// </summary>
		private void updateXSDAttributeXML()
		{
			if(m_masterDescriptionDoc == null)
			{
				if(!loadMasterDescription()) return;
			}
			loadXSDAttributeXML();
			for(int i = 0; i < m_attributeTypeItemList.Count; i++)
			{
				XmlElement item = (XmlElement)m_attributeTypeItemList.Item(i);
				string strFlPlnCmpntAttrDefColNm = item.GetAttribute("Name");
				string type = findTypeForFlPlnCmpntAttrDefColNm(strFlPlnCmpntAttrDefColNm);
				if(type.Length != 0)
				{
					item.SetAttribute("Type",type);
				}
				else
				{
					item.SetAttribute("Type","UNKNOWN");
				}
			}
			m_attributeTypeDoc.Save(m_AttributeTypeFile);
		}

		private string findTypeForFlPlnCmpntAttrDefColNm(string strFlPlnCmpntAttrDefColNm)
		{
			string retVal = "";
			string xpath = "Component/Attribute[@FlPlnCmpntAttrDefColNm=\"" + strFlPlnCmpntAttrDefColNm + "\"]";
			XmlNodeList attrList = m_masterDescriptionRoot.SelectNodes(xpath);
			if(attrList != null)
			{
				if(attrList.Count == 0)
				{
					util.log("WARNING " + strFlPlnCmpntAttrDefColNm + " data Not Found"); 
				}
				bool bOk = true;
				string temp = null;
				string test;
				for(int i = 0; i < attrList.Count; i++)
				{
					test = attrList.Item(i).Attributes.GetNamedItem("FlPlnCmpntAttrDefDatTyp").Value;
					if(i != 0)
					{
						if(!temp.Equals(test))
						{
							bOk = false;
							retVal += "," + test;
						}
					}
					else
					{
						retVal = test;
					}
					temp = test;
				}
				if(!bOk)
				{
					util.log("WARNING " + strFlPlnCmpntAttrDefColNm + " contains more than one data type: " + retVal);
				}
			}
			return retVal;
		}
		
		/// <summary>
		/// Open Folder Browser Dialog to select the location
		/// of the ...\API_Reference folder.
		/// </summary>
		private bool selectAPIRefLocation()
		{
			folderBrowserDialog1.Description = "Select the location of the API_Reference folder";
			folderBrowserDialog1.SelectedPath = "C:\\Dev\\IBM\\Slammer\\EngineWorkspace\\API_Reference";
			if(folderBrowserDialog1.ShowDialog(this) == DialogResult.OK)
			{
				m_apiRefLocation = folderBrowserDialog1.SelectedPath;
			} 
			else
			{
				return false;
			}

			return true;
		}
		
		/// <summary>
		/// Adds nCount components for all allowed targets to the specified File Plan Component.
		/// See Autogen.
		/// </summary>
		/// <param name="nFlPlnCmpntID">The Id of the component that the targets will be added to.</param>
		/// <param name="nFlPlnViewID">The View Id of the component that the targets will be added to.</param>
		/// <param name="nCount">The number of components (of each type) to add.</param>
		private void addFilePlanComponents(long nFlPlnCmpntID, long nFlPlnViewID, long nCount)
		{
			string xml = m_session.getFilePlanComponentController().getAllowedTargetList(nFlPlnCmpntID,nFlPlnViewID);
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.LoadXml(xml);
				XmlNodeList itemNodes = doc.GetElementsByTagName("Item");
				for(int i = 0; i < itemNodes.Count; i++)
				{
					string strFlPlnCmpntDefNm = itemNodes.Item(i).Attributes.GetNamedItem("FlPlnCmpntDefNm").Value;
					string strFlPlnCmpntDefTgtID = itemNodes.Item(i).Attributes.GetNamedItem("FlPlnCmpntDefTgtID").Value;
					
					string compDefList = "<List><Item FlPlnCmpntDefID='" + strFlPlnCmpntDefTgtID + "'></Item></List>";
					string test = m_session.getNavigationController().getTargetList(nFlPlnViewID,nFlPlnCmpntID,0,compDefList,0,0);
					XmlDocument doc2 = new XmlDocument();
					try
					{
						doc2.LoadXml(test);
						XmlNodeList itemNodes2 = doc2.GetElementsByTagName("Item");
						int begin = itemNodes2.Count + 1; 
						
						long nFlPlnCmpntDefTgtID = System.Convert.ToInt64(strFlPlnCmpntDefTgtID);

						for(int j = begin; j < begin + nCount; j++)
						{
							string name = strFlPlnCmpntDefNm + (j);
							xml =	"<Fields>" +
								"<Field FlPlnCmpntAttrDefColNm='FlPlnCmpntNm'>" + name + "</Field>" +
								"<Field FlPlnCmpntAttrDefColNm='FlPlnCmpntTtl'>" + name + "</Field>" +
								"</Fields>";
						
							try
							{
								m_session.getFilePlanComponentController().addFilePlanComponent(nFlPlnCmpntDefTgtID,xml,nFlPlnCmpntID);
							}
							catch(System.Web.Services.Protocols.SoapException s)
							{
								util.handleSoapException(s,false);
							}
						}
					}
					catch(XmlException e3)
					{
						util.logXMLErr(e3, "getTargetList(" + nFlPlnViewID + ", " + nFlPlnCmpntID + ",0, " + compDefList + ",0,0)");
					}
				}
			}
			catch(XmlException e4)
			{
				util.logXMLErr(e4, "getAllowedTargetList(" + nFlPlnCmpntID + ", " + nFlPlnViewID + ")");
			}

		}
		#endregion

		#region FormAndMenuEvents
		
		/// <summary>
		/// Update the Attribute Name to Type mapping file
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuUpdateAttributeTypeFile_Click(object sender, System.EventArgs e)
		{
			updateXSDAttributeXML();
		}

		/// <summary>
		/// Get the location of the ...\API_Reference folder.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuJavadocsLocation_Click(object sender, System.EventArgs e)
		{
			selectAPIRefLocation();
		}

		/// <summary>
		/// Logs into the Records Manager
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void Form1_Load(object sender, System.EventArgs e)
		{
			doLogin(this.m_isHostLogin);
		}

		/// <summary>
		/// Logs into the Records Manager using login method
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuLogin_Click(object sender, System.EventArgs e)
		{
			doLogin(false);
		}

		/// <summary>
		/// Logs into the Records Manager using hostLogin method
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuHostLogin_Click(object sender, System.EventArgs e)
		{
			doLogin(true);
		}

		/// <summary>
		/// Executes Login
		/// </summary>
		/// <param name="isHostLogin"></param>
		private void doLogin(bool isHostLogin)
		{
			try
			{
				if(initClientSession(isHostLogin))
				{
					//Clear Treeview
					tvFilePlan.Nodes.Clear();
					CommonTasks.DrawFilePlan(m_session,tvFilePlan);
				}
			}
			catch(System.Net.WebException we)
			{
				m_bLoggedIn = false;
				util.logWebException(we);
			}
			catch(System.Web.Services.Protocols.SoapException s)
			{
				m_bLoggedIn = false;
				util.handleSoapException(s,false);
			}
		}

		//public static int EXPAND_LISTS = 1; 
		//public static int EXPAND_LISTS_AND_PROFILE_FIELDS = 3; 
		//public static int EXPAND_LISTS_AND_PROFILE_FIELDS_NO_FLS = 4; 
		//public static int NONE = 0; 
		//public static int PROFILE_FIELDS = 2; 

		/// <summary>
		/// Test Code - use this area to try things out.
		/// Display results on Log Message page.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuTestCode_Click(object sender, System.EventArgs e)
		{

			if(MessageBox.Show(this,"TEST CODE HERE. This Mehtod will ERASE all logged messages! Do you wish to continue?","WARNING",MessageBoxButtons.YesNo,MessageBoxIcon.Warning) == DialogResult.No)
			{
				return;
			}

			String xml = "";

			if(m_lastSelectedNode == null)
			{
				MessageBox.Show("Select an item from the File Plan before running test!");
				return;
			}

			//This will get the information for the last selected Tree Node
			IRMComponent irm = (IRMComponent)m_lastSelectedNode.Tag;
			long fpcId = irm.FlPlnCmpntID;
			long defId = irm.FlPlnCmpntDefID;


//			string strQuery =
//			"<Fields>" +
//				"<Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntDefID\">0</Field>" +
//				"<Field FlPlnCmpntAttrDefColNm=\"UsrGrpOwnerID\">-1</Field>" + 
//				"<Field FlPlnCmpntAttrDefColNm=\"QueryType\">1</Field>" + 
//				"<Field FlPlnCmpntAttrDefColNm=\"OutputType\">1</Field>" + 
//				"<Field FlPlnCmpntAttrDefColNm=\"ShowBatchList\">0</Field>" + 
//				"<Field FlPlnCmpntAttrDefColNm=\"QueryImage\">" +
//					"<![CDATA[ <ReportTemplate><Version>4.1.3.000</Version><ReportLayout/><Scope/><BlockSize>40</BlockSize><BlockStart>0</BlockStart><Distinct>0</Distinct><Query><Condition/><SortFields/><OutputFields><OutputField>TS_FlPlnCmpnt.FlPlnCmpntNm</OutputField><OutputField>TS_FlPlnCmpnt.FlPlnCmpntCode</OutputField><OutputField>TS_FlPlnCmpnt.FlPlnCmpntTtl</OutputField></OutputFields></Query></ReportTemplate>]]>" + 
//				"</Field>" +
//			"</Fields>";

			//xml = m_session.getQueryController().runQuery(strQuery, (int)IRMConstants.QueryResultOptions.NORMAL);
			//textBox1.Text = xml;

			//textBox1.Text = m_session.getUserController().getUserById(-1,true);
			//textBox1.Text = m_session.getTemplateController().getNewComponent(21,0,0,EXPAND_LISTS_AND_PROFILE_FIELDS);

			//string xml = m_session.getTemplateController().getNewComponent(21,0,0,EXPAND_LISTS_AND_PROFILE_FIELDS);
			//XmlDocument doc = new XmlDocument();
			//doc.LoadXml(xml);

			//string xpath = "/Field[@FlPlnCmpntAttrDefColNm='TS_Location']"; 

			//textBox1.Text = doc.SelectSingleNode(xpath).Value;

			//string newTitle = "Look Ma I changed my title at " + System.DateTime.Now;
			//m_session.getFilePlanComponentController().setFilePlanComponentField(148,"FlPlnCmpntTtl", newTitle);

			//textBox1.Text = m_session.getQueryController().getQueryById(22);

			//int nContainerDefId = 21;
			
			//string query = 
			//"<Fields>" +
			//	"<Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntDefID\">" + nContainerDefId + "</Field>" +
			//	"<Field FlPlnCmpntAttrDefColNm=\"QueryImage\">" +
			//		"<![CDATA[<ReportTemplate><Version>4.1.1</Version><ReportLayout/><Scope/><BlockSize>0</BlockSize><BlockStart>0</BlockStart><Distinct>0</Distinct><Query><Condition/><SortFields/><OutputFields><OutputField>TS_FlPlnCmpnt.FlPlnCmpntNm</OutputField></OutputFields></Query><GetEligibleFilePlanComponents><ReferenceDateFrom/><ReferenceDateTo/><IncludeDisposalAuthority>0</IncludeDisposalAuthority><EligibleForCutoff>0</EligibleForCutoff><EligibleForClose>0</EligibleForClose><EligibleForTransition>1</EligibleForTransition><LifeCycleQueryType>1</LifeCycleQueryType><PhaseId/></GetEligibleFilePlanComponents></ReportTemplate>]]>" +
			//	"</Field>" +
			//"</Fields>";

			/*
			 * JMS message can only be seen by user who originated the report or a member of the Administrators group.
			 * We could save the taskId returned from the beginTask method or found in the XML result from the
			 * getStatusListByTaskId method that the originator of the report calls.
			 * This task Id could be used to retrieve the results of the life cycle query allowing other users to view the results.
			*/

			//string taskId = "414d51205741535f54505457454947415dc1b54120000601";
			//textBox1.Text = m_session.getBatchController().getStatusListByTaskId(taskId);

			//com.ibm.gre.engine.types.LogTypes 
			//int ERROR_LOG = 2; 
			//int META_DATA_LOG = 4; 
			//int RESULT_LOG = 3; 
			//int TRANSITION_LOG = 1; 

			//textBox1.Text = m_session.getBatchController().getTaskResultsByID(taskId,RESULT_LOG,0,0);

			//textBox1.Text = m_session.getQueryController().getReportColumnsList(21);

			///////////////////// IRMQuery tests ///////////////////
			
			//string xmlQuery = m_session.getQueryController().getQueryById(81);
			
			//IRMQuery irmQuery = new IRMQuery(xmlQuery,m_session, "",true, OptionalFields.NO_FIELDS);
			//textBox1.Text = irmQuery.QueryImage;
			//textBox1.Text = irmQuery.ComponentName;
			//textBox1.Text = irmQuery.Query;
			//textBox1.Text = irmQuery.Test;

			//irmQuery.setFieldValue("TS_FlPlnCmpnt_Cst21.TS_TransitNumber_1","345");

			//textBox1.Text = irmQuery.getFieldValue("TS_FlPlnCmpnt_Cst21.TS_TransitNumber_1");
			
			//textBox1.Text = irmQuery.Condition;

			//xmlQuery = irmQuery.ModifiedQuery;
			//textBox1.Text = m_session.getQueryController().runQuery(xmlQuery,2);
			
			//textBox1.Clear();
			//for(int i = 0; i < irmQuery.LineCount; i++)
			//{
			//	Line line = irmQuery.getLine(i);
			//	string test = line.Caption + "  DatTyp: " + line.FlPlnCmpntAttrDefDatTyp + " PickListId: " + line.PickListId + " FieldOperator" + line.LineType;

			//	log(test + "\r\n");
			//}
			
			//textBox1.Text = "this.m_serverNamePort: " + this.m_serverNamePort + "\r\nthis.m_databaseJNDIName: " + this.m_databaseJNDIName;

			
			//long crossReferenceViewID = 4;
			//string sourceList = 
			//	"<List>" +
			//		"<Item FlPlnCmpntID=\"465\" FlPlnViewID=\"" + crossReferenceViewID + "\" />" + 
			//		"<Item FlPlnCmpntID=\"466\" FlPlnViewID=\"" + crossReferenceViewID + "\" />" + 
			//		"<Item FlPlnCmpntID=\"844\" FlPlnViewID=\"" + crossReferenceViewID + "\" />" + 
			//		"<Item FlPlnCmpntID=\"524\" FlPlnViewID=\"" + crossReferenceViewID + "\" />" + 
			//	"</List>";
			//long targetID = 1213;	
			
			//m_session.getFilePlanComponentController().addSource(targetID,sourceList);

			//textBox1.Text = m_session.getFilePlanComponentController().getPermissionList(fpcId,defId,"",0,0);
			//textBox1.Text = m_session.getSecurityController().getExtendedPermissionList(irm.FlPlnCmpntID,"",0,0);
			//textBox1.Text = m_session.getNavigationController().getSourceList(0,irm.FlPlnCmpntID,0,0);

//			string newComponentXml = 
//			"<Fields>" +
//				"<Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntNm\">Component added with WierdoField</Field>" +
//				"<Field FlPlnCmpntAttrDefColNm=\"WierdoField\">Nonexsitent</Field>" +
//			"</Fields>";

			//m_session.getFilePlanComponentController().addFilePlanComponent(defId,newComponentXml,irm.FlPlnCmpntID);

			//xml = m_session.getTemplateController().getFilePlanComponent(fpcId,0,(int)IRMConstants.TemplateOptions.EXPAND_LISTS_AND_PROFILE_FIELDS);
			//textBox1.Text = xml;

			//xml = m_session.getFilePlanComponentController().getMembersOfSet(fpcId,2,0,0);
			//textBox1.Text = xml;

			//22
			//xml = m_session.getTemplateController().getComponentStructure(22);
			//textBox1.Text = xml;

			//textBox1.Text = m_session.getEngineVersion();
			
			//textBox1.Text = "User ID:" +  m_session.getUserController().getUserId();

			//textBox1.Text = "Functin Access Mask:" + m_session.getUserController().getUserActionMask();

			
			//textBox1.Text = m_session.getUserController().getUserByName("QAHost","testuser",true);


//			string userListXML =
//			"<List>" +
//			"<Item UsrNm=\"testuser\" XtUsrID=\"testuser\" />" + 
//			"</List>";
//			
//			string userFieldXML =
//			"<Fields>" +
//			"<Field FlPlnCmpntAttrDefColNm=\"RecHsID\">1</Field>" + 
//			"<Field FlPlnCmpntAttrDefColNm=\"FnctActnMsk\">161</Field>" + 
//			"<Field FlPlnCmpntAttrDefColNm=\"SecClsID\">2</Field>" + 
//			"<Field FlPlnCmpntAttrDefColNm=\"IsActive\">1</Field>" + 
//			"<Field FlPlnCmpntAttrDefColNm=\"SecDescLst\" />" +
//			"</Fields>";
//
			//m_session.getUserController().addUserList(userListXML,userFieldXML);

			//defId

			//textBox1.Text = m_session.getComponentDesignController().getDefaultListByDefinitionId(defId);

			//textBox1.Text = m_session.getComponentDesignController().getProfileListByComponentDefinitionId(defId);

			//long profileID = -300;
			
			//long nFlPlnCmpntDfltID = 81;

			//textBox1.Text = m_session.getTemplateController().getNewComponent(defId,profileID,nFlPlnCmpntDfltID,(int)IRMConstants.TemplateOptions.NONE);

			String xml1 = "<Fields><Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntNm\">TestFile[";
			String xml2 = "]</Field></Fields>";
			xml = xml1 + "WarmUp" + xml2;
//
//			m_session.getFilePlanComponentController().addFilePlanComponent(defId,xml,fpcId);
//
//			int count = 1000;
//			DateTime start = System.DateTime.Now;
//			for(int i = 0; i < count; i++)
//			{
//				xml = xml1 + i + xml2;
//				m_session.getFilePlanComponentController().addFilePlanComponent(defId,xml,fpcId);
//			}
//			DateTime end = System.DateTime.Now;
//			
//			System.TimeSpan delta = end - start;
//
//			String str = ".NET perftest - Elapsed Time in Minutes: " + delta.Minutes + ":" + delta.Seconds + "." + delta.Milliseconds + " for " + count + " declarations";
//			textBox1.Text = str;
//			MessageBox.Show(str);

		}

		private void mnuAttrCntrlMaskInterigator_Click(object sender, System.EventArgs e)
		{
			InputForm frm = new InputForm("Enter Attribute Control mask value","Resulting constants");

			if(frm.ShowDialog(this) == DialogResult.OK)
			{
				Regex numberPattern = new Regex(@"^\d+$");
				if(!numberPattern.IsMatch(frm.TextBox1)) {
					MessageBox.Show("You should enter one integer for the control mask value");
				}
				else 
				{
					int n = System.Convert.ToInt32(frm.TextBox1);				
					string str = IRMAttrCntrlMsk.toStringRepresentation(n);
					frm.TextBox2 = str;
					frm.ShowDialog(this);
				}
			}
		}


		/// <summary>
		/// Fired when a tab selection is made.
		/// Refresh the Life Cycle results if that page is selected.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void tabControl2_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			if(tabControl2.SelectedTab.Name == "tabLifeCycleReports")
			{
				if(m_bLoggedIn)
				{
					DrawLifeCycleReports();
				}
			}
		}

		#endregion

		#region LifeCycleReportListEvents
		
		/// <summary>
		/// Responds to the Life Cycle Reports List View context menu
		/// command - "View Results"
		/// Displays the results of the Life Cycle Query for the selected item.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuViewLifeCycleResults_Click(object sender, System.EventArgs e)
		{
			int count = lvLifeCycleReports.SelectedItems.Count;
			if(count > 0)
			{
				string taskId = null;
				IEnumerator items = lvLifeCycleReports.SelectedItems.GetEnumerator();
				items.MoveNext();
				ListViewItem lvi = (ListViewItem)items.Current;
				taskId = (string)lvi.Tag;

				string xmlResultList = m_session.getBatchController().getTaskResultsByID(taskId,(int)IRMConstants.LogTypes.RESULT_LOG,0,0);
				IRMLifeCycleResultList rl = new IRMLifeCycleResultList(xmlResultList);

				//Make sure that only the following columns will be displayed
				rl.AddColumn("NameFullPath");
				rl.AddColumn("Conflict");
				rl.AddColumn("NextEligiblePhaseName");
				rl.AddColumn("NextEligiblePhaseDate");

				LifeCycleResultListForm lcrlFrm = new LifeCycleResultListForm(taskId, m_session,this,rl,0,true);
				lcrlFrm.Show();

			}
		}

		/// <summary>
		/// Responds to the Life Cycle Reports List View context menu
		/// command - "Delete"
		/// Deletes the selected batch operation and all associated operations.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuDeleteLifeCycleResults_Click(object sender, System.EventArgs e)
		{
			int count = lvLifeCycleReports.SelectedItems.Count;
			if(count > 0)
			{
				string msg = "Delete Selected Tasks and all related tasks?";
				IEnumerator items = lvLifeCycleReports.SelectedItems.GetEnumerator();
				if(count > 1)
				{
					if(MessageBox.Show(this,msg,"Life Cycle Operations",MessageBoxButtons.YesNo) == DialogResult.No)
					{
						return;
					}
				}
				while(items.MoveNext())
				{
					ListViewItem lvLifeCycleReportsi = (ListViewItem)items.Current;
					if(count == 1)
					{
						msg = "Delete Task: " + lvLifeCycleReportsi.Text + " and all related tasks";
						if(MessageBox.Show(this,msg,"Life Cycle Operations",MessageBoxButtons.YesNo) == DialogResult.No)
						{
							return;
						}
					}
					try
					{
						
						string taskName = lvLifeCycleReportsi.Text;

						string xml = m_session.getBatchController().getTaskList(m_session.getUserController().getUserId());
						IRMList list = new IRMList(xml,"TaskID","TaskName");

						//Remove all items that are TRANSITION_FILE_PLAN_COMPONENTS or GET_ELIGIBLE_FILE_PLAN_COMPONENTS tasks
						//and have the same name as the current task (cheap trick)
						for(int i = list.Count - 1; i > -1; i--)
						{
							int taskType = Convert.ToInt32(list.getAttributeValue(i,"Type"));
							if(taskType == (int)IRMConstants.AsynchronousTaskType.TRANSITION_FILE_PLAN_COMPONENTS
								|| taskType == (int)IRMConstants.AsynchronousTaskType.GET_ELIGIBLE_FILE_PLAN_COMPONENTS)
							{
								if(taskName == list.Caption(i))
								{
									m_session.getBatchController().deleteFinishedTask(list.PrimaryKey(i));
								}
							}
						}
					}
					catch(System.Net.WebException we)
					{
						util.logWebException(we);
						return;
					}
					catch(System.Web.Services.Protocols.SoapException s)
					{
						util.handleSoapException(s,false); //false indicates we will not check for time ou here
						util.showSoapException(s);
						return;
					}
					lvLifeCycleReportsi.Remove();
				}
			}			
		}

		/// <summary>
		/// Responds to the Life Cycle Reports List View context menu
		/// command - "Transition"
		/// Transitions the list of File Plan Components in the selected
		/// Life Cycle Results item.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void mnuTransitionLifeCycleResults_Click(object sender, System.EventArgs e)
		{
			int count = lvLifeCycleReports.SelectedItems.Count;
			if(count > 0)
			{
				string strTransferContentDir = CommonTasks.getEngineConfigValue(m_session,util,"sysTransferRecordsContentDir");
				if(strTransferContentDir == "")
				{
					MessageBox.Show("Transferred Records Content Directory value has not been set. Set this value in the System Configuration of the IBM DB2 Records Manager Administrator.");
					return;	
				}

				string strKeepDisposedMetaDataFlag = CommonTasks.getEngineConfigValue(m_session,util,"sysKeepMetaDataOnAccession");				
				if(strKeepDisposedMetaDataFlag == "")
				{
					strKeepDisposedMetaDataFlag = "0";
				}
				
				string strTransferMetaDataDir = CommonTasks.getEngineConfigValue(m_session,util,"sysTransferRecordsMetaDataDir");				
				if(strTransferMetaDataDir == "")
				{
					MessageBox.Show("Transferred Records Meta Data Directory value has not been set. Set this value in the System Configuration of the IBM DB2 Records Manager Administrator.");
					return;	
				}

				string msg = "Transition ALL Selected Life Cycle Result Lists?";
				IEnumerator items = lvLifeCycleReports.SelectedItems.GetEnumerator();
				if(count > 1)
				{
					if(MessageBox.Show(this,msg ,"Life Cycle Operations",MessageBoxButtons.YesNo) == DialogResult.No)
					{
						return;
					}
				}
				string taskId;
				while(items.MoveNext())
				{
					ListViewItem lvLifeCycleReportsi = (ListViewItem)items.Current;
					taskId = (string)lvLifeCycleReportsi.Tag;
					if(count == 1)
					{
						msg = "Transition the Life Cycle Result List: " + lvLifeCycleReportsi.Text;
						if(MessageBox.Show(this,msg,"Life Cycle Operations",MessageBoxButtons.YesNo) == DialogResult.No)
						{
							return;
						}
					}
					try
					{
						//TODO this is a real cheap way to co-ordinate tasks
						//Just use the same unique name for all
						string taskName = lvLifeCycleReportsi.Text;
						string xml =
						"<Fields>" +
                            "<Field FlPlnCmpntAttrDefColNm=\"SourceTaskID\">" + taskId + "</Field>" + 
							"<Field FlPlnCmpntAttrDefColNm=\"TransferContentDir\">" + strTransferContentDir + "</Field>" +
							"<Field FlPlnCmpntAttrDefColNm=\"KeepDisposedMetaDataFlag\">" + strKeepDisposedMetaDataFlag + "</Field>" +
							"<Field FlPlnCmpntAttrDefColNm=\"TransferMetaDataDir\">" + strTransferMetaDataDir + "</Field>" + 
						"</Fields>";
						
						m_session.getBatchController().beginTask((int)IRMConstants.AsynchronousTaskType.TRANSITION_FILE_PLAN_COMPONENTS,xml,taskName,"");
					}
					catch(System.Net.WebException we)
					{
						util.logWebException(we);
						return;
					}
					catch(System.Web.Services.Protocols.SoapException s)
					{
						util.handleSoapException(s,false); //false indicates we will not check for time out here
						util.showSoapException(s);
						return;
					}
				}
			}			
		}

		/// <summary>
		/// Responds to the Life Cycle Reports List View context menu
		/// command - "Show Transition Results..."
		/// Shows the status values for the Transitions executed on the selected Life Cycle Results item.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void menuViewTransitionResults_Click(object sender, System.EventArgs e)
		{
			int count = lvLifeCycleReports.SelectedItems.Count;
			if(count > 0)
			{
				IEnumerator items = lvLifeCycleReports.SelectedItems.GetEnumerator();
				items.MoveNext();
				ListViewItem lvi = (ListViewItem)items.Current;
				
				string taskName = lvi.Text;

				string xml = m_session.getBatchController().getTaskList(m_session.getUserController().getUserId());
				IRMList list = new IRMList(xml,"TaskID","TaskName");

				//Remove all items that are not TRANSITION_FILE_PLAN_COMPONENTS tasks
				//and do not have the same name as the current task (cheap trick)
				for(int i = list.Count - 1; i > -1; i--)
				{
					int taskType = Convert.ToInt32(list.getAttributeValue(i,"Type"));
					if(taskType != (int)IRMConstants.AsynchronousTaskType.TRANSITION_FILE_PLAN_COMPONENTS)
					{
						list.removeItem(i);
					}
					else
					{
						if(taskName != list.Caption(i))
						{
							list.removeItem(i);
						}
					}
				}
				if(list.Count == 0)
				{
					MessageBox.Show("No Transition results for this item");
				}
				else
				{
					TransitionResultsForm frm = new TransitionResultsForm(m_session,this,list);
					frm.Show();
				}
			}
		}

		#endregion
		
		#region LifeCycleReportListMethods

		/// <summary>
		/// Draws the status items for all batch Life Cycle Reports
		/// for the current user.
		/// </summary>
		private void DrawLifeCycleReports()
		{
			lvLifeCycleReports.Items.Clear();
			lvLifeCycleReports.Columns.Clear();
			long nUserId = m_session.getUserController().getUserId();
			string xml = m_session.getBatchController().getTaskList(nUserId);
			IRMList list = new IRMList(xml,"TaskID","TaskName");

			float totalWidth = lvLifeCycleReports.Width;
			
			int width = (int)(.4 * totalWidth);
			lvLifeCycleReports.Columns.Add("Task Name",width,HorizontalAlignment.Left);
			width = (int)(.1 * totalWidth);
			lvLifeCycleReports.Columns.Add("Status",width,HorizontalAlignment.Left);
			width = (int)(.2 * totalWidth);
			lvLifeCycleReports.Columns.Add("Start Time",width,HorizontalAlignment.Left);
			width = (int)(.1 * totalWidth);
			lvLifeCycleReports.Columns.Add("% Done",width,HorizontalAlignment.Left);
			width = (int)(.2 * totalWidth);
			lvLifeCycleReports.Columns.Add("Status Time",width,HorizontalAlignment.Left);

			for(int i = 0; i < list.Count; i++)
			{
				ListViewItem item = new ListViewItem(list.Caption(i));
				
				int type = Convert.ToInt32(list.getAttributeValue(i,"Type"));

				//Only Add GET_ELIGIBLE_FILE_PLAN_COMPONENTS task type
				if(type == (int)IRMConstants.AsynchronousTaskType.GET_ELIGIBLE_FILE_PLAN_COMPONENTS)
				{
					string strStatus = list.getAttributeValue(i,"Status");
					int nStatus = Convert.ToInt32(strStatus);
					item.SubItems.Add(Util.makeStatusMsg(nStatus));
					item.SubItems.Add(list.getAttributeValue(i,"StartTime"));
					item.SubItems.Add(strStatus);
					item.SubItems.Add(list.getAttributeValue(i,"StatusTime"));

					item.Tag = list.PrimaryKey(i);
					lvLifeCycleReports.Items.Add(item);
				}
			}

		}
		#endregion

		#region MemberVariables

		private string m_apiRefLocation = null;

		private string m_masterDescFile;
		private XmlDocument m_masterDescriptionDoc = null;
		private XmlElement m_masterDescriptionRoot = null;
		private XmlNodeList m_masterDescriptionComponentList = null;

		private string m_AttributeTypeFile;
		private XmlDocument m_attributeTypeDoc = null;
		private XmlNodeList m_attributeTypeItemList = null;

		private LookUpForm m_LookUp = null;

		private Util util;

		private ClientSession m_session;

		private bool m_isHostLogin;
		private string m_serverNamePort;
		private string m_databaseName;
		private string m_databaseJNDIName;
		private string m_language;
		private string m_extUserId;
		private string m_userName;
		private bool m_bLoggedIn = false;

		private TreeNode m_lastSelectedNode = null;

		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.OpenFileDialog openFileDialog1;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.ContextMenu cntxtMnuListView;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem mnuEditListItem;
		private System.Windows.Forms.MenuItem mnuViewListItem;
		private System.Windows.Forms.MenuItem mnuDeleteListItem;
		private System.Windows.Forms.ContextMenu cntxtMnuTreeView;
		private System.Windows.Forms.MenuItem mnuEditTreeItem;
		private System.Windows.Forms.MenuItem mnuViewTreeItem;
		private System.Windows.Forms.MenuItem mnuDeleteTreeItem;
		private System.Windows.Forms.MenuItem menuItem8;
		private System.Windows.Forms.MenuItem mnuSearchTreeItem;
		private System.Windows.Forms.MenuItem menuItem6;
		private System.Windows.Forms.MenuItem mnuAutoGen;
		private System.Windows.Forms.TabControl tabControl2;
		private System.Windows.Forms.TabPage tabFilePlanExplorer;
		private System.Windows.Forms.StatusBar statusBar1;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.TabPage tabTextPage;
		private System.Windows.Forms.TextBox textBox1;
		private System.Windows.Forms.TreeView tvFilePlan;
		private System.Windows.Forms.Splitter splitter2;
		private System.Windows.Forms.ColumnHeader columnHeaderName;
		private System.Windows.Forms.ColumnHeader columnHeaderComponent;
		private System.Windows.Forms.ColumnHeader columnHeaderTitle;
		private System.Windows.Forms.ListView lvComponents;
		private System.Windows.Forms.MenuItem mnuTestCode;
		private System.Windows.Forms.MenuItem menuLifeCycleQuery;
		private System.Windows.Forms.ImageList imageList1;
		private System.Windows.Forms.MenuItem mnuAddTreeItemTarget;
		private System.Windows.Forms.MenuItem mnuAddListItemTarget;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.MenuItem mnuRefreshTreeItem;

		private System.Windows.Forms.TabPage tabLifeCycleReports;
		private System.Windows.Forms.ListView lvLifeCycleReports;
		private System.Windows.Forms.ContextMenu cntxtMnuBatchList;
		private System.Windows.Forms.MenuItem mnuViewLifeCycleResults;
		private System.Windows.Forms.MenuItem mnuDeleteLifeCycleResults;
		private System.Windows.Forms.MenuItem mnuTransitionLifeCycleResults;
		private System.Windows.Forms.MenuItem menuViewTransitionResults;
		private System.Windows.Forms.MenuItem mnuMigrateLitsItems;
		private System.Windows.Forms.MenuItem mnuUnMigrateListItems;
		private System.Windows.Forms.MenuItem mnuLogin;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem mnuJavadocsLocation;
		private System.Windows.Forms.FolderBrowserDialog folderBrowserDialog1;
		private System.Windows.Forms.MenuItem mnuUpdateAttributeTypeFile;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem mnuAttrCntrlMaskInterigator;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem mnuMetadataTreeItem;
		private System.Windows.Forms.MenuItem mnuUserPermissionsTreeItem;
		private System.Windows.Forms.MenuItem mnuExtendedPermissionsTreeItem;
		private System.Windows.Forms.MenuItem mnuMetadaPlus;
		private System.Windows.Forms.MenuItem menuItem7;
		private System.Windows.Forms.MenuItem mnuTemplNoneTreeItem;
		private System.Windows.Forms.MenuItem mnuTemplExpandLstTreeItem;
		private System.Windows.Forms.MenuItem mnuTemplProfilesTreeItem;
		private System.Windows.Forms.MenuItem mnuTemplListsProfilesTreeItem;
		private System.Windows.Forms.MenuItem mnuHostLogin; 
		

		#endregion

	}
}
