using System;
using System.Xml;

namespace com.ibm.gre
{
	/// <summary>
	/// Extends the IRMResultList class with methods for
	/// determining root and conflict nodes.
	/// </summary>
	public class IRMLifeCycleResultList : IRMResultList
	{
		public IRMLifeCycleResultList(IRMQuery irmQuery) :	base(irmQuery)
		{
		}

		public IRMLifeCycleResultList(string resultList) : base(resultList)
		{
		}

		/// <summary>
		/// Used to determine if the specified Item is a Root
		/// Component in a Life Cycle Report.
		/// </summary>
		/// <param name="index">Index of the Item.</param>
		/// <returns>Returns true if the specified item is a Root Component.</returns>
		public bool isRoot(int index)
		{
			XmlAttribute attr = (XmlAttribute)m_Items.Item(index).Attributes.GetNamedItem("isRoot");
			if(attr == null)
			{
				return false;
			}
			return (attr.Value == "1");
		}

		/// <summary>
		/// Used to determine if the Conflict value
		/// for the specified item is non zero.
		/// </summary>
		/// <param name="index">Index of the Item.</param>
		/// <returns>Returns true if the Conflict value is non zero.</returns>
		public bool hasConflict(int index)
		{
			XmlAttribute attr = (XmlAttribute)m_Items.Item(index).Attributes.GetNamedItem("Conflict");
			if(attr == null)
			{
				return false;
			}
			return (attr.Value != "0");
		}

	}
}
