<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:exsl="http://exslt.org/common"
				extension-element-prefixes="exsl">

	<xsl:template match="/">
		<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">

			<!-- defines page layout -->
			<fo:layout-master-set>
				<fo:simple-page-master master-name="simple" page-width="21cm" page-height="27.9cm" margin-top="0.5cm" margin-bottom="0.5cm" margin-left="0.5cm" margin-right="0.5cm">
					<fo:region-body />
				</fo:simple-page-master>
			</fo:layout-master-set>
			
			<xsl:variable name="counts" select="count(/ResultList/Data/List/Item)"/>
			<xsl:variable name="pageSize" select="count(exsl:node-set($iteration)/List/Item)"/>
			
			<xsl:if test="$counts = 0">
				<xsl:call-template name="blankPage"/>
			</xsl:if>
			
			<xsl:if test="$counts > 0">
				<xsl:for-each select="/ResultList/Data/List/Item">				
					<xsl:if test="((position()-1) mod $pageSize) = 0">
						<xsl:call-template name="processPageSequence">
							<xsl:with-param name="curPos" select="position()"/>
							<xsl:with-param name="allCounts" select="$counts"/>
							<xsl:with-param name="curPageDataListItem" select="."/>							
						</xsl:call-template>				
					</xsl:if>
				</xsl:for-each>
		  </xsl:if>
		</fo:root>
	</xsl:template>
	
	<xsl:template name ="blankPage">
			<!-- defines the content -->
			<fo:page-sequence master-reference="simple">
				<fo:flow flow-name="xsl-region-body">

					<fo:block font-size="14pt" font-weight="bold" text-align="center">Empty Content</fo:block>
					
				</fo:flow>
			</fo:page-sequence>	
	</xsl:template>	

	<xsl:template name ="processPageSequence">
	<xsl:param name="curPos"/>
	<xsl:param name="allCounts"/>
	<xsl:param name="curPageDataListItem"/>
			<!-- defines the content -->
			<fo:page-sequence master-reference="simple">

				<fo:flow flow-name="xsl-region-body">
					<fo:block font-size="16pt" font-family="Helvetica" font-weight="bold" line-height="26pt" space-after.optimum="12pt" text-align="center">Folder Report</fo:block>
				
					<fo:table border-top="solid black" border-bottom="solid black" border-left="solid black" border-right="solid black" font-size="6.5pt">
						<!-- Column width setting  -->
						<fo:table-column column-width="7cm" />
						<fo:table-column column-width="3.5cm" />
						<fo:table-column column-width="5.2cm" />
						<fo:table-column column-width="1.5cm" />
						<fo:table-column column-width="1.5cm" />						
						
						<fo:table-header>
							<!-- Report Headers   -->
							<fo:table-row>
								<!-- File Plan Location -->
								<fo:table-cell border-top="solid black" border-bottom="solid black" border-right="solid black" padding="3pt">
									<fo:block>
									<xsl:value-of select="/ResultList/MetaData/List/Item[@FlPlnCmpntAttrDefColNm='NameFullPath']/@FlPlnCmpntAttrDefCapt"/>
									</fo:block>
								</fo:table-cell>

								<!-- Folder Name -->
								<fo:table-cell border-top="solid black" border-bottom="solid black" border-right="solid black" padding="3pt">
									<fo:block >
									<xsl:value-of select="/ResultList/MetaData/List/Item[@FlPlnCmpntAttrDefColNm='TS_FlPlnCmpnt.FlPlnCmpntNm']/@FlPlnCmpntAttrDefCapt"/>
									</fo:block>
								</fo:table-cell>
								
								<!-- Folder Title -->
								<fo:table-cell border-top="solid black" border-bottom="solid black" border-right="solid black" padding="3pt">
									<fo:block >
									<xsl:value-of select="/ResultList/MetaData/List/Item[@FlPlnCmpntAttrDefColNm='TS_FlPlnCmpnt.FlPlnCmpntTtl']/@FlPlnCmpntAttrDefCapt"/>
									</fo:block>
								</fo:table-cell>
								
								<!-- Life Cycle Code Name -->
								<fo:table-cell border-top="solid black" border-bottom="solid black" border-right="solid black" padding="3pt">
									<fo:block >
									<xsl:value-of select="/ResultList/MetaData/List/Item[@FlPlnCmpntAttrDefColNm='TS_LfCclCd.LfCclCdNm']/@FlPlnCmpntAttrDefCapt"/>
									</fo:block>
								</fo:table-cell>

								<!-- Disposal Authority -->
								<fo:table-cell border-top="solid black" border-bottom="solid black" padding="3pt">
									<fo:block >
									<xsl:value-of select="/ResultList/MetaData/List/Item[@FlPlnCmpntAttrDefColNm='TS_FlPlnCmpnt.DsplAthrtyID']/@FlPlnCmpntAttrDefCapt"/>
									</fo:block>
								</fo:table-cell>

							</fo:table-row>													
						</fo:table-header>
						
						<fo:table-body>

							<xsl:for-each select="exsl:node-set($iteration)/List/Item">
								<xsl:variable name="j" select="position() - 1"/>
								<xsl:if test="$j = 0">
									<xsl:call-template name="processDataItem">
										<xsl:with-param name="curDataListItem"  select="$curPageDataListItem" />
										<xsl:with-param name="pos" select="$curPos" />
										<xsl:with-param name="allCounts" select="$allCounts"/>							
									</xsl:call-template>	
								</xsl:if>
								
								<xsl:if test="$j &gt; 0">
									<xsl:call-template name="processDataItem">
										<xsl:with-param name="curDataListItem"  select="$curPageDataListItem/following-sibling::*[$j]" />
										<xsl:with-param name="pos" select="$curPos + $j" />
										<xsl:with-param name="allCounts" select="$allCounts"/>							
									</xsl:call-template>	
								</xsl:if>							
							</xsl:for-each>				

						</fo:table-body>																							
					</fo:table>

				</fo:flow>
			</fo:page-sequence>
		</xsl:template>
		
		<xsl:template name="processDataItem">
				<xsl:param name="curDataListItem"/>		
								<fo:table-row>
										<!-- File Plan Location -->
										<fo:table-cell border-right="solid black" padding="2pt">
											<fo:block text-align="start">
												<xsl:value-of select="$curDataListItem/@NameFullPath" />		 
											</fo:block>
										</fo:table-cell>

										<!-- Folder Name -->
										<fo:table-cell border-right="solid black"   padding="2pt">
											<fo:block text-align="start">
												<xsl:value-of select="$curDataListItem/@TS_FlPlnCmpnt.FlPlnCmpntNm" />
											</fo:block>
										</fo:table-cell>
										
										<!-- Folder Title -->
										<fo:table-cell border-right="solid black"   padding="2pt">
											<fo:block text-align="start">
												<xsl:value-of select="$curDataListItem/@TS_FlPlnCmpnt.FlPlnCmpntTtl" />
											</fo:block>
										</fo:table-cell>
										
										<!-- Life Cycle Code -->
										<fo:table-cell border-right="solid black" padding="2pt">
											<fo:block text-align="start">
												<xsl:value-of select="$curDataListItem/@TS_LfCclCd.LfCclCdNm" />												
											</fo:block>
										</fo:table-cell>

										<!-- Disposal Authority -->
										<fo:table-cell border-right="solid black" padding="2pt">
											<fo:block text-align="start">
												<xsl:value-of select="$curDataListItem/@TS_FlPlnCmpnt.DsplAthrtyID" />												
											</fo:block>
										</fo:table-cell>

									</fo:table-row>

	</xsl:template>
	
	<!--Page size congiguration used for iteration, default is 20-->
	<xsl:variable name="iteration">

		<List>
			<Item count="1"/>			
			<Item count="2"/>
			<Item count="3"/>			
			<Item count="4"/>
			<Item count="5"/>			
			<Item count="6"/>
			<Item count="7"/>			
			<Item count="8"/>
			<Item count="9"/>			
			<Item count="10"/>
			<Item count="1"/>			
			<Item count="2"/>
			<Item count="3"/>			
			<Item count="4"/>
			<Item count="5"/>			
			<Item count="6"/>
			<Item count="7"/>			
			<Item count="8"/>
			<Item count="9"/>			
			<Item count="20"/>
		</List>

	</xsl:variable>			

</xsl:stylesheet>