/**
 Licensed Materials - Property of IBM
'Restricted Materials of IBM'
5724-I58
 Copyright IBM Corp. 2001, 2005 All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.gre.dod.extension;

import java.io.*;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.Date;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.Message.RecipientType;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.log4j.Logger;
import org.apache.xml.serialize.Method;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import com.ibm.gre.engine.ejb.facade.ClientControllerEJB;
import com.ibm.gre.engine.ejb.facade.ConfigurationControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.ConfigurationControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.DefinitionControllerEJB;
import com.ibm.gre.engine.ejb.facade.DefinitionControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.DefinitionControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.FilePlanDesignControllerEJB;
import com.ibm.gre.engine.ejb.facade.FilePlanDesignControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.FilePlanDesignControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.LifeCycleDesignControllerEJB;
import com.ibm.gre.engine.ejb.facade.LifeCycleDesignControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.LifeCycleDesignControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.LoginManagerEJB;
import com.ibm.gre.engine.ejb.facade.LoginManagerEJBHome;
import com.ibm.gre.engine.ejb.facade.NavigationControllerEJB;
import com.ibm.gre.engine.ejb.facade.NavigationControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.NavigationControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.QueryControllerEJB;
import com.ibm.gre.engine.ejb.facade.QueryControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.QueryControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.UserControllerEJB;
import com.ibm.gre.engine.ejb.facade.UserControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.UserControllerEJBLocalHome;
import com.ibm.gre.engine.ejb.facade.SchemaMappingControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.SchemaMappingControllerEJBLocalHome;
import com.ibm.gre.engine.extension.FilePlanComponentListener;
import com.ibm.gre.engine.options.DualNamingOptions;
import com.ibm.gre.engine.util.BusinessComponentsJNDINames;
import com.ibm.gre.engine.util.BusinessLayerError;
import com.ibm.gre.engine.util.ConversionUtil;
import com.ibm.gre.engine.util.DAOLayerErrors;
import com.ibm.gre.engine.util.DispositionTypes;
import com.ibm.gre.engine.util.ExtensionParamKeys;
import com.ibm.gre.engine.util.QueryResultOptions;
import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.RecordManagerExceptionTypes;
import com.ibm.gre.engine.util.ServiceLocator;
import com.ibm.gre.engine.util.ValidationUtil;
import com.ibm.gre.engine.util.XMLUtil;
import com.ibm.gre.engine.value.DBConfig;
import com.ibm.gre.engine.value.StateMachine;
import com.ibm.gre.engine.value.StateMachineKeys;
import com.ibm.gre.engine.value.User;
import com.ibm.gre.engine.util.ComponentDefinitionTypes;
import com.ibm.gre.engine.util.AttributeDataTypes;
import com.ibm.gre.engine.types.SecurityClassificationEventTypes;

/** 
 * @version 	2.1
 * @author 		tianxj
 * Purpose: Separate the single transition exported file into two, permanent and temporary files.
 * Add a new handle to new event afterTransitionExportToSpecificSchemaEvent.
 * 
 * 
 * @version 	2.0
 * @author 		tianxj		
 * Purpose:
 * 1. Remove the support to custom project permissions while has been implemented as a built-in feature Custom Security Descriptor. 
 * Thus, the handles for events of beforeAddEvent, afterAddEvent, afterUpdateEvent and afterGetByIDEvent were removed.
 * 2. Add a new handle to event afterTransitionEvent, handle the DOD requirement of the life cycle phase transition of a file plan component can trigger another file plan component to be eligible for cut off. 
 * 
 * 
 * @version 	1.0
 * @author		waisea
 * 
 * Purpose: purpose handle Dod document custom project permissions
 * Handles the following events:
 *	beforeAddEvent
 * 	afterAddEvent
 *	afterUpdateEvent
 * 	afterGetByIDEvent  
 * 
 * IRM Configuration Requirements
 * ------------------------------
 * TS_User - must have TS_Project custom attribute of type String and multivalued enabled
 * optionally TS_Group too can have TS_Project custom attributes.
 * 
 * Custom component must have following custom attributes: 
 * 		1) TS_Project type String NON-multivalue droplist
 * 
 * The custom file plan component profile must contain both TS_Project custom attribute.
 * 
 * Within the package is a file called ApplicationResources.properties, it contains most
 * of the default values for the custom attriubtes stated above. In addition, you should be
 * able to modify the login user name and password using that file. This login user must be
 * administrator because it need to modify component's securities.
 * 
 * 
 * Building this project
 * ---------------------
 * 1) Make sure you have the lastest IRMAPI.jar and log4j-1.2.7.jar in your project root directory.
 * Change your Java Build Path Libraries to reference them.
 * 
 * 2) Export project as a jar, call it IRMDocumentExtension.jar include
 * package com.ibm.gre.dod.extension, do not include IRMAPI.jar and log4j-1.2.7.jar these
 * will be picked up from the engine class path when the class is loaded.
 * 
 * 
 *  Tips for running within IDE with Engine projects:
 *  ----------------
 * 	1) Modify Mocha (EAR) 
 * 		Deployment Descriptor - Module
 * 		Add (Project Utility JARs) to include IRMDocumentExtension.jar
 * 		Save it
 *  2) Modify MocharEJB MANIFEST.MF to include IRMDocumentExtension.jar as Dependencies
 *  3) To build IRMDocumentExtension, add project references to MochaEJB and MochaJava
 * 
 * ---------------- 
 * Revision History
 * ----------------
 * Date				Author		Changes
 * -------------------------------------
 * July 25, 2003		waisea		Created
 * Aug 26, 2003			waisea		Added beforeAddEvent handle new Dod case, and added external resource file for customization.
 * Sept 12, 2003		waisea		Get user controller as Administrator because calling userController.getGroupListByUserId requires User/Group function access
 * May 13, 2004			Tom Chen	Updated for Trafalgar.(i.e. removed the checking for Supplemental Markings)
 * July 09, 2007		Tianxj		Remove the support for custom project. Add support for phase transition trigger cut off.
 * 
 */
public class DocumentListener implements FilePlanComponentListener {
	// hard coded default user name and password
	private static String ADMIN_USER_NAME = "administrator";
	private static String ADMIN_PASSWORD = "cronos";

	// error constants	
	private static int E_EXTSN_BASE = 800000;
	private static int E_EXTSN_LOGIN_FAILED = E_EXTSN_BASE + 1; // failed login - required Administrator/cronos
	private static int E_EXTSN_NO_CLNT_CNTRLR = E_EXTSN_BASE + 2; // failed to create adminClientController
	private static int E_EXTSN_NO_QRY_CNTRLR = E_EXTSN_BASE + 10; // failed to create query controller	
	private static int E_EXTSN_NO_DECLASSIFICATION_REVIEW = E_EXTSN_BASE +100; //need declassification review

	// Application Resource Constants
	final private static String APP_RES_FILE_NAME = "ApplicationResources.properties";
	final private static String APP_RES_LOGIN_NAME = "login.user.name";
	final private static String APP_RES_LOGIN_PW = "login.user.pw";
	
	final private static String APP_RES_CST_EMAIL_ADDRESS = "custom.field.email_address";
	final private static String APP_RES_CST_INFORM_ON_SUPERCEDE = "custom.field.inform_on_supercede";
	final private static String	APP_RES_MAIL_SMTP_HOST = "mail.smtp.host";
	final private static String	APP_RES_MAIL_FROM_ADDRESS = "mail.from.address";
	
	// Application Resource Constants for transition trigger cut off
	final private static String APP_TRANS_FOLDER_DEF_NAME = "transition.folder.definition.name";
	final private static String APP_TRANS_FOLDER_LFC_NAME = "transition.folder.lifecycle.code.name";
	final private static String APP_TRANS_FOLDER_LFC_INSTRUCTION_NAME = "transition.folder.lifecycle.instruction.name";
	final private static String APP_TRANS_FOLDER_LFC_PHS_NAME = "transition.folder.lifecycle.phase.name";
	final private static String APP_TRANS_RECORD_DEF_NAME = "transition.record.definition.name";
	final private static String APP_TRANS_LNK_VW_NAME = "transition.link.view.name";
	final private static String APP_TRANS_LNKED_RECORD_DEF_NAME = "transition.linked.record.definition.name";
	final private static String APP_TRANS_LNKED_RECORD_LFC_NAME = "transition.linked.record.lifecycle.code.name";
	final private static String APP_TRANS_LNKED_RECORD_LFC_DATE_COLNAME = "transition.linked.record.lifecycle.date.column.name";
	//Transition Export to Schema Constants
	final private static String APP_TRAN_EXOPT_SCHEMA_NAME="transition.export.schema.name";
	final private static String APP_TRAN_EXOPT_CAT_NAME="transition.export.category.definition.name";
	final private static String APP_TRAN_EXOPT_CAT_NARA_INDICATOR="transition.export.category.transfer2nara.attribute.name";
	final private static String APP_TRAN_EXOPT_CAT_PERMANENT_INDICATOR="transition.export.category.permanent.attribute.name";
	final private static String APP_TRAN_EXOPT_PERM_DIR="transition.export.nara.permanent.file.directory";
	final private static String APP_TRAN_EXOPT_TEMP_DIR="transition.export.nara.temporary.file.directory";
	private final static String fileSeparator = System.getProperty("file.separator");	
	//Declassification Review
	final private static String APP_DECA_LFC_NAME = "declassification.lifecycle.code.name";
	final private static String APP_DECA_LFC_REV_ATTR_NAME="declassification.lifecycle.code.need.review.attribute.name";
	final private static String APP_DECA_LFC_INSTRUCTION_NAME="declassification.lifecycle.instruction.name";
	final private static String APP_DECA_LFC_PHS_NAME="declassification.lifecycle.phase.name";
	final private static String APP_DECA_FOLDER_NAME="declassification.folder.definition.name";
	final private static String APP_DECA_RECORD_NAME="declassification.record.definition.name";
	final private static String APP_DECA_EMAIL_ATTR_NAME="declassification.user.email.attribute.name";
	final private static String APP_DECA_ADMIN_EMAIL="declassification.admin.email.address";
	final private static String APP_DECA_EMAIL_BODY="declassification.notify.mail.body";
	final private static String APP_DECA_EMAIL_TITLE="declassification.notify.mail.title";
	
	
	// Application Resouce Constants for error string
	private static String E_RES_LOGIN_BAD_USR_PW = "error.login.bad_user_pw";
	private static String E_RES_NO_DECLASSIFICATION_REVIEW = "error.no.declassification.review";
	
	private Properties prop; // ApplicationResources.properties
	// loaded from application resources
	private String adminUserName;
	private String adminPassword;
	private String cstEmailAddress;
	private String cstInformOnSupercede;
	
	private String mailSmtpHost;
	private String mailFromAddress;
	
	private String transFolderDefinitionName;
	private String transFolderLifeCycleCodeName;
	private String transFolderLifeCycleCodeInstructionName;
	private String transFolderLifeCyclePhaseName;
	private String transRecordDefinitionName;
	private String transLinkViewName;
	private String transLinkedRecordDefinitionName;
	
	private String transExportSchemaName;
	private String transExportCategoryName;
	private String transExportCategoryNARAIndicator;
	private String transExportCategoryPermanentIndicator;
	private String transExportPermanentDir;
	private String transExportTemporaryDir;	
	
	private String declassifyLifeCycleCodeName;
	private String declassifyLifeCycleNeedReviewAttributeName;
	private String declassifyLifeCycleInstructionName;
	private String declassifyLifeCyclePhaseName;
	private String declassifyFolderName;
	private String declassifyRecordName;
	private String declassifyEmailAttrName;
	private String declassifyAdminEmail;
	
	
	private InitialContext context = null ;
	
	// client controller that logs in as administrator - for changePermission operation
	private ClientControllerEJB adminClientController; 
	
	private static Logger logger = Logger.getLogger("EXTENSION");
	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeAddEvent(EventObject)
	 */
	public void beforeAddEvent(EventObject e) throws RecordManagerException {
		
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterAddEvent(EventObject)
	 */
	public void afterAddEvent(EventObject e) throws RecordManagerException {

	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeDeleteEvent(EventObject)
	 */
	public void beforeDeleteEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterDeleteEvent(EventObject)
	 */
	public void afterDeleteEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByBarCodeEvent(EventObject)
	 */
	public void beforeGetByBarCodeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByBarCodeEvent(EventObject)
	 */
	public void afterGetByBarCodeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeUpdateEvent(EventObject)
	 */
	public void beforeUpdateEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterUpdateEvent(EventObject)
	 */
	public void afterUpdateEvent(EventObject e) throws RecordManagerException {

		
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByIDEvent(EventObject)
	 */
	public void beforeGetByIDEvent(EventObject e) throws RecordManagerException {
		//logger.info("starting beforeGetByIDEvent ...");
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByIDEvent(EventObject)
	 */
	public void afterGetByIDEvent(EventObject e) throws RecordManagerException {

	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSetFieldEvent(EventObject)
	 */
	public void beforeSetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSetFieldEvent(EventObject)
	 */
	public void afterSetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetFieldEvent(EventObject)
	 */
	public void beforeGetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetFieldEvent(EventObject)
	 */
	public void afterGetFieldEvent(EventObject e)
		throws RecordManagerException {
	}
	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeTransitionEvent(EventObject)
	 */
	public void beforeTransitionEvent(EventObject e) throws RecordManagerException {
		logger.info("beforeTransitionEvent - BEGIN");
		
		this.initApplicationResources();
		
		//transition trigger cut off
		transitionTriggerCutOff(e);
		
		//Declassification review check.
		declassificationReview(e);
				
		logger.info("beforeTransitionEvent - END");
	}
	
	public void transitionTriggerCutOff(EventObject e) throws RecordManagerException {
		logger.info("transitionTriggerCutoff - BEGIN");
				
		//retrieve the parameters
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		long fpcId = (Long) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_ID);		
		long fpcPhaseId = (Long) map.get(ExtensionParamKeys.PHASE_ID);
		long fpcInstructionId = (Long) map.get(ExtensionParamKeys.INSTRUCTION_ID);
		
		//get all APIs
		DefinitionControllerEJBLocal defBean = this.getDefinitionControllerEJBLocal(stateMachine);
		FilePlanComponentControllerEJBLocal fpcBean = this.getFilePlanComponentControllerLocal(stateMachine);
		LifeCycleDesignControllerEJBLocal lfcBean = this.getLifeCycleDesignControllerEJBLocal(stateMachine);
		NavigationControllerEJBLocal navBean = this.getNavigationControllerEJBLocal(stateMachine);
		FilePlanDesignControllerEJBLocal fpcDesignBean = this.getFilePlanDesignControllerEJBLocal(stateMachine);
		
		//get current fpc definition and lfc code
		String fpc = fpcBean.getFilePlanComponentById(fpcId);
		Document fpcDoc = XMLUtil.getDocument(fpc);
		long fpcDefId = XMLUtil.getOptionalFieldLongValue(fpcDoc, "FlPlnCmpntAttrDefColNm", "FlPlnCmpntDefID");
		long fpcLfcCodeId = XMLUtil.getOptionalFieldLongValue(fpcDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdID");
		
		if(0 ==  fpcLfcCodeId){
			logger.error("No record found for id: "+fpcId); return;			
		}
			
		//get folder def id
		String folderDef = defBean.getComponentDefinitionByName(this.transFolderDefinitionName);
		Document folderDefDoc = XMLUtil.getDocument(folderDef);
		Node folderDefNode = XMLUtil.getNodeByXPath(folderDefDoc, "/List/Item");
		long folderDefId = XMLUtil.getLongValueFromXMLAttribute(folderDefNode, "FlPlnCmpntDefID");
		
		if(0 == folderDefId){
			logger.error("No folder name found: "+this.transFolderDefinitionName); return;
		}
			
		//If current fpc is not a folder, return
		if(fpcDefId != folderDefId) return;
			
		//get folder life cycle code
		String folderLfcCode = null;
		long folderLfcCodeId = 0;
		try {
			folderLfcCode = lfcBean.getLifeCycleManagementCodeByName(this.transFolderLifeCycleCodeName);
			Document folderLfcCodeDoc = XMLUtil.getDocument(folderLfcCode);
			folderLfcCodeId = XMLUtil.getOptionalFieldLongValue(folderLfcCodeDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdID");
			
		} catch (RecordManagerException e1) {
			//just bypass LFC not found exception
			logger.error(e1);
		}
		
		if(0 == folderLfcCodeId){
			logger.error("No folder life cycle code found: "+this.transFolderLifeCycleCodeName); return;
		}
			
		//If current fpc lfc code not same to folder lfc code, return 
		if(fpcLfcCodeId != folderLfcCodeId) return;
		
		String folderInstruction = null;
		long folderLfcInstructionId = 0;
		try {
			folderInstruction = lfcBean.getLifeCycleManagementCodeInstrByCodeInstrName(folderLfcCodeId,this.transFolderLifeCycleCodeInstructionName);
			Document folderInstructionDoc = XMLUtil.getDocument(folderInstruction);
			folderLfcInstructionId = XMLUtil.getOptionalFieldLongValue(folderInstructionDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdInstrID");
		} catch (RecordManagerException e1) {
			//just bypass LFC not found exception
			logger.error(e1);
		}
		
		//if the lfc instruction id is not same to folder instruction id, return
		if(0 == folderLfcInstructionId){
			logger.error("No folder life cycle instruction id found: "+transFolderLifeCycleCodeInstructionName); return;
		}
		
		if(folderLfcInstructionId != fpcInstructionId) return;
		
		long folderLfcPhaseId = 0;
		
		if(DispositionTypes.ACCESSION_NAME.equals(transFolderLifeCyclePhaseName)){
			folderLfcPhaseId = DispositionTypes.ACCESSION;
		}else if(DispositionTypes.DESTROY_NAME.equals(transFolderLifeCyclePhaseName)){
			folderLfcPhaseId = DispositionTypes.DESTROY;
		}else if(DispositionTypes.EXPORT_NAME.equals(transFolderLifeCyclePhaseName)){
			folderLfcPhaseId = DispositionTypes.EXPORT;
		}else if(DispositionTypes.REVIEW_NAME.equals(transFolderLifeCyclePhaseName)){
			folderLfcPhaseId = DispositionTypes.REVIEW;
		}else{		
			String folderLfcPhase = lfcBean.getLifeCyclePhaseDefinitionByName(this.transFolderLifeCyclePhaseName);
			Document folderLfcPhaseDoc = XMLUtil.getDocument(folderLfcPhase);
			folderLfcPhaseId = XMLUtil.getOptionalFieldLongValue(folderLfcPhaseDoc, "FlPlnCmpntAttrDefColNm", "LfCclPhsID");
			
			if(0 == folderLfcPhaseId){				
				logger.error("No folder life cycle phase found: "+this.transFolderLifeCyclePhaseName); return;
			}
		}
		
		//If current fpc lfc phase not same to required folder lfc phase, return 
		if(fpcPhaseId != folderLfcPhaseId) return;
			
		//find record definition under the folder
		String recordDef = defBean.getComponentDefinitionByName(this.transRecordDefinitionName);
		Document recordDefDoc = XMLUtil.getDocument(recordDef);
		Node recordDefNode = XMLUtil.getNodeByXPath(recordDefDoc, "/List/Item");
		long recordDefId = XMLUtil.getLongValueFromXMLAttribute(recordDefNode, "FlPlnCmpntDefID");
			
		//find all records under current folder
		String filter = "<List><Item FlPlnCmpntDefID='"+recordDefId+"'/></List>";
			
		String recordsUnderFolder = navBean.getTargetListWithFilter(-1, fpcId, 0, filter, 0, 0);
		Document recordsUnderFolderDoc = XMLUtil.getDocument(recordsUnderFolder);
		NodeList recodeNodes = XMLUtil.getNodeListByXPath(recordsUnderFolderDoc, "/List/Item");
			
		//If there is no record under folder, just return
		if(recodeNodes.getLength() == 0){
			logger.error("No record found under folder: "+fpcId); return;
		}
						
		//get the link view id
		String linkView = null;
		long linkViewId = 0;
		
		try{
			linkView = fpcDesignBean.getFilePlanViewByName(this.transLinkViewName);
			Document linkViewDoc = XMLUtil.getDocument(linkView);
			linkViewId = XMLUtil.getOptionalFieldLongValue(linkViewDoc, "FlPlnCmpntAttrDefColNm", "FlPlnViewID");
		} catch (RecordManagerException e1) {
			//just bypass LFC not found exception
			logger.error(e1);
		}
		
		if(0 == linkViewId){
			logger.error("No link view found for view name: "+transLinkViewName); return;
		}
		
		//get the linked record definition id
		String linkedRecordDef = defBean.getComponentDefinitionByName(this.transLinkedRecordDefinitionName);
		Document linkedRecordDefDoc = XMLUtil.getDocument(linkedRecordDef);
		Node linkedRecordDefNode = XMLUtil.getNodeByXPath(linkedRecordDefDoc, "/List/Item");
		long linkedRecordDefId = XMLUtil.getLongValueFromXMLAttribute(linkedRecordDefNode, "FlPlnCmpntDefID");
		String linkedRecordFilter = "<List><Item FlPlnCmpntDefID='"+linkedRecordDefId+"'/></List>";
			
		//for each record under folder, find its linked record
		Node recordNode = null;
		for(int i = 0; (recordNode = recodeNodes.item(i)) != null; i++){
			long recordId = XMLUtil.getLongValueFromXMLAttribute(recordNode, "FlPlnCmpntID");				
			String linkedRecords = navBean.getTargetList(linkViewId, recordId, 0, linkedRecordFilter, 0, 0);
			Document linkedRecordsDoc = XMLUtil.getDocument(linkedRecords);
			NodeList linkedRecodeNodes = XMLUtil.getNodeListByXPath(linkedRecordsDoc, "/List/Item");
			if(linkedRecodeNodes.getLength() == 0){
				logger.error("No linked record found for record: "+recordId);
				continue;
			}
			//set the life cycle date of the linked record
			Node linkedRecordNode = null;
			for(int j = 0; (linkedRecordNode = linkedRecodeNodes.item(j)) != null; j++){
				long linkedRecordId = XMLUtil.getLongValueFromXMLAttribute(linkedRecordNode, "FlPlnCmpntID");
				String linkedRecord = fpcBean.getFilePlanComponentById(linkedRecordId);
				Document linkedRecordDoc = XMLUtil.getDocument(linkedRecord);
				long linkedRecordLfcCodeId = XMLUtil.getOptionalFieldLongValue(linkedRecordDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdID");
				if(linkedRecordLfcCodeId == 0){
					logger.error("No life cycle found for linked record: "+linkedRecordId);
					continue;
				}
				String lfcDateList = lfcBean.getLifeCycleDateList(linkedRecordLfcCodeId);
				Document lfcDateListDoc = XMLUtil.getDocument(lfcDateList);
				Node linkedRecordLfcDateNode = XMLUtil.getNodeByXPath(lfcDateListDoc, "/List/Item[@FlPlnCmpntDefID="+linkedRecordDefId+"]");
				String linkedRecordLfcDate = XMLUtil.getOptionalStringValueFromAttribute(linkedRecordLfcDateNode, "FlPlnCmpntAttrDefColNm");
				
				if(null == linkedRecordLfcDate || linkedRecordLfcDate.equals("")){
					logger.error("No life cycle date found for record: "+linkedRecordId);
					continue;
				}
				
				//if the life cycle date is not empty, just bypass
				String lfcDate = XMLUtil.getOptionalFieldStringValue(linkedRecordDoc, "FlPlnCmpntAttrDefColNm", linkedRecordLfcDate);
				if(lfcDate == null || lfcDate.equals("")){ 
					fpcBean.setFilePlanComponentField(linkedRecordId, linkedRecordLfcDate, ConversionUtil.dateToString(new Date()));
				}
			}
		}
		
		logger.info("transitionTriggerCutOff - END");
	}
	
	private void declassificationReview(EventObject e) throws RecordManagerException{
		logger.info("declassificationReview - BEGIN");
		//retrieve the parameters
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		User user = (User)stateMachine.getAttribute(StateMachineKeys.USER);		
		long fpcId = (Long) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_ID);		
		long fpcPhaseId = (Long) map.get(ExtensionParamKeys.PHASE_ID);
		long fpcInstructionId = (Long) map.get(ExtensionParamKeys.INSTRUCTION_ID);
		
		//get all APIs
		DefinitionControllerEJBLocal defBean = this.getDefinitionControllerEJBLocal(stateMachine);
		FilePlanComponentControllerEJBLocal fpcBean = this.getFilePlanComponentControllerLocal(stateMachine);
		LifeCycleDesignControllerEJBLocal lfcBean = this.getLifeCycleDesignControllerEJBLocal(stateMachine);
		UserControllerEJBLocal usrBean = this.getUserControllerEJBLocal(stateMachine);
		NavigationControllerEJBLocal navBean = this.getNavigationControllerEJBLocal(stateMachine);
		
		//get record and folder def id
		long folderDefId = 0;
		long recordDefId = 0;
		long lfcCodeId = 0;
		long lfcInstructionId = 0;	
		long lfcPhaseId = 0;
		
		try {
			//check if need review attribute exists
			String lfcAttributes = defBean.getAttributeDefinitionByCompDefID(ComponentDefinitionTypes.LIFE_CYCLE_CODE);
			Document lfcAttributesDoc = XMLUtil.getDocument(lfcAttributes);
			Node needReviewNode = XMLUtil.getNodeByXPath(lfcAttributesDoc, "/List/Item[@FlPlnCmpntAttrDefColNm=\""+this.declassifyLifeCycleNeedReviewAttributeName+"\"]");
			
			if(needReviewNode == null) return;
			long needReviewDataType = XMLUtil.getLongValueFromXMLAttribute(needReviewNode, "FlPlnCmpntAttrDefDatTyp");
			if(needReviewDataType != AttributeDataTypes.BOOLEAN) return;
		} catch (RecordManagerException e1) {
			logger.error(e1); return;
		}
		
		try{
			//check if the lfc exists
			String lfcCode = lfcBean.getLifeCycleManagementCodeByName(this.declassifyLifeCycleCodeName);
			Document lfcCodeDoc = XMLUtil.getDocument(lfcCode);
			lfcCodeId = XMLUtil.getOptionalFieldLongValue(lfcCodeDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdID");
			
			if(lfcCodeId == 0){
				logger.error("Lifecycle code "+declassifyLifeCycleCodeName+" doesn't exists."); return;
			}
			
			//check the need review
			boolean needReview = XMLUtil.getOptionalFieldBooleanValue(lfcCodeDoc, "FlPlnCmpntAttrDefColNm", declassifyLifeCycleNeedReviewAttributeName);
			
			if(!needReview){ 
				logger.error("Lifecycle code "+declassifyLifeCycleCodeName+" doesn't need declassification review.");return;			
			}
		} catch (RecordManagerException e1) {
			logger.error(e1); return;
		}
			
		try{
			//check if the lfc instruction exits
			String lfcInstruction = lfcBean.getLifeCycleManagementCodeInstrByCodeInstrName(lfcCodeId,this.declassifyLifeCycleInstructionName);
			Document lfcInstructionDoc = XMLUtil.getDocument(lfcInstruction);
			lfcInstructionId = XMLUtil.getOptionalFieldLongValue(lfcInstructionDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdInstrID");
			
			if(lfcInstructionId == 0){
				logger.info("Lifecycle instruction "+declassifyLifeCycleInstructionName+" doesn't exists."); return;
			}
			
			//check if the lfc phase exists
			if(DispositionTypes.ACCESSION_NAME.equals(declassifyLifeCyclePhaseName)){
				lfcPhaseId = DispositionTypes.ACCESSION;
			}else if(DispositionTypes.DESTROY_NAME.equals(declassifyLifeCyclePhaseName)){
				lfcPhaseId = DispositionTypes.DESTROY;
			}else if(DispositionTypes.EXPORT_NAME.equals(declassifyLifeCyclePhaseName)){
				lfcPhaseId = DispositionTypes.EXPORT;
			}else if(DispositionTypes.REVIEW_NAME.equals(declassifyLifeCyclePhaseName)){
				lfcPhaseId = DispositionTypes.REVIEW;
			}else{		 
				String folderLfcPhase = lfcBean.getLifeCyclePhaseDefinitionByName(this.declassifyLifeCyclePhaseName);
				Document folderLfcPhaseDoc = XMLUtil.getDocument(folderLfcPhase);
				lfcPhaseId = XMLUtil.getOptionalFieldLongValue(folderLfcPhaseDoc, "FlPlnCmpntAttrDefColNm", "LfCclPhsID");
				
				if(0 == lfcPhaseId){				
					logger.error("No life cycle phase found: "+this.declassifyLifeCyclePhaseName); return;
				}
			}
		} catch (RecordManagerException e1) {
			logger.error(e1); return;
		}
		
		try{

			//check if folder def exists
			String folderDef = defBean.getComponentDefinitionByName(this.declassifyFolderName);
			Document folderDefDoc = XMLUtil.getDocument(folderDef);
			Node folderDefNode = XMLUtil.getNodeByXPath(folderDefDoc, "/List/Item");
			folderDefId = XMLUtil.getLongValueFromXMLAttribute(folderDefNode, "FlPlnCmpntDefID");
			
			if(folderDefId == 0){
				logger.error("No fpc definition for "+declassifyFolderName);return;
			}

			//check if record def exists
			String recordDef = defBean.getComponentDefinitionByName(this.declassifyRecordName);
			Document recordDefDoc = XMLUtil.getDocument(recordDef);
			Node recordDefNode = XMLUtil.getNodeByXPath(recordDefDoc, "/List/Item");
			recordDefId = XMLUtil.getLongValueFromXMLAttribute(recordDefNode, "FlPlnCmpntDefID");
			
		} catch (RecordManagerException e1) {
			logger.error(e1); return;
		}			
		
		//get current fpc definition and lfc code
		String fpc = fpcBean.getFilePlanComponentById(fpcId);
		Document fpcDoc = XMLUtil.getDocument(fpc);
		long fpcDefId = XMLUtil.getOptionalFieldLongValue(fpcDoc, "FlPlnCmpntAttrDefColNm", "FlPlnCmpntDefID");
		long fpcLfcCodeId = XMLUtil.getOptionalFieldLongValue(fpcDoc, "FlPlnCmpntAttrDefColNm", "LfCclCdID");		
				
		//check if current fpc is not a folder or record
		if(fpcDefId != folderDefId && fpcDefId != recordDefId){
			logger.error("not matched folder def id");return;
		}
		
		if(0 ==  fpcLfcCodeId){
			logger.error("No lifecycle code found for file plan component: "+fpcId); return;			
		}
		
		if(fpcLfcCodeId != lfcCodeId || fpcInstructionId != lfcInstructionId || fpcPhaseId != lfcPhaseId){
			logger.error("No right lifecycle code, instruction, phase combination for component: "+fpcId); return;		
		}
		
		NodeList reviewEventNodeList = XMLUtil.getNodeListByXPath(fpcDoc, "/Fields/Field[@FlPlnCmpntAttrDefColNm=\"SecClsInfo\"]/Fields/Field[@FlPlnCmpntAttrDefCapt=\"Event List (History)\" ]/Fields/Field[@FlPlnCmpntAttrDefColNm=\"SecClsEvntTyp\"]");
		
		if(reviewEventNodeList == null || reviewEventNodeList.getLength() == 0){
			logger.error("No classification event find for fpcId: "+fpcId+".");
			String fpcPath = navBean.getFullPathById(fpcId, -1, DualNamingOptions.BY_NAME);
			if(fpcDefId == folderDefId){
				//send email
				String usr = usrBean.getUserById(user.getUserID(), false);
				Document usrDoc = XMLUtil.getDocument(usr);
				String emailAddr = XMLUtil.getOptionalFieldStringValue(usrDoc, "FlPlnCmpntAttrDefColNm", this.declassifyEmailAttrName);
				if(emailAddr != null && !emailAddr.equals("")){ 
					String mailBody = getMessage(APP_DECA_EMAIL_BODY, "\""+fpcPath+"\"");
					String mailTitle = getMessage(APP_DECA_EMAIL_TITLE);
					try {
						sendMail(this.declassifyAdminEmail, new String[]{emailAddr},mailTitle,mailBody);
					} catch (Exception e1) {
						logger.error(e1);
					}
				}
				
			}
			RecordManagerException rme = new RecordManagerException(E_EXTSN_NO_DECLASSIFICATION_REVIEW, getMessage(E_RES_NO_DECLASSIFICATION_REVIEW,"\""+fpcPath+"\""), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC);
			throw rme;
		}
				
		//if current node is a folder
			
		int listLen = reviewEventNodeList.getLength();
		boolean reviewed = false;
		for(int i=0; i<listLen; i++){
			Node reviewEventNode = reviewEventNodeList.item(i);
			long eventType = XMLUtil.getLongValue(reviewEventNode);
			if(eventType == SecurityClassificationEventTypes.REVIEW){
				reviewed = true;
				break;
			}
		}
			
		if(reviewed){
			logger.error("fpcId: "+fpcId+" has been reviewed.");
		}else{
			
			String fpcPath = navBean.getFullPathById(fpcId, -1, DualNamingOptions.BY_NAME);
			if(fpcDefId == folderDefId){
				//send email
				String usr = usrBean.getUserById(user.getUserID(), false);
				Document usrDoc = XMLUtil.getDocument(usr);
				String emailAddr = XMLUtil.getOptionalFieldStringValue(usrDoc, "FlPlnCmpntAttrDefColNm", this.declassifyEmailAttrName);
				if(emailAddr != null && !emailAddr.equals("")){ 
					String mailBody = getMessage(APP_DECA_EMAIL_BODY, "\""+fpcPath+"\"");
					String mailTitle = getMessage(APP_DECA_EMAIL_TITLE);
					try {
						sendMail(this.declassifyAdminEmail, new String[]{emailAddr},mailTitle,mailBody);
					} catch (Exception e1) {
						logger.error(e1);
					}
				}
				
			}
			RecordManagerException rme = new RecordManagerException(E_EXTSN_NO_DECLASSIFICATION_REVIEW, getMessage(E_RES_NO_DECLASSIFICATION_REVIEW,"\""+fpcPath+"\""), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC);
			throw rme;
		}
		logger.info("declassificationReview - END");
	}
	
	private void sendMail (String fromAddress, String[] toAddress, String subject, String body) throws Exception{
		
		Properties props = new Properties ();
		props.put("mail.smtp.host", mailSmtpHost);
   
   		Session sendMailSession = Session.getDefaultInstance(props);
   		
   		MimeMessage mailMessage = new MimeMessage(sendMailSession);
   		mailMessage.setSubject(subject);

		mailMessage.setText(body);

   		InternetAddress addressFrom = new InternetAddress(fromAddress);
   		mailMessage.setFrom(addressFrom);
   		
   		for (int i=0 ; i < toAddress.length ; i++){
   			mailMessage.addRecipient(RecipientType.TO,new InternetAddress(toAddress[i]));
   		}
   		java.util.Enumeration enum1 = mailMessage.getAllHeaderLines();
   		while(enum1.hasMoreElements()){
   			Object obj = enum1.nextElement();
   			System.out.println(obj.toString());
   		}
		Transport.send(mailMessage);
				
	}
	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterTransitionEvent(EventObject)
	 */
	public void afterTransitionEvent(EventObject e) throws RecordManagerException{
		
	}	


	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSupercedeEvent(java.util.EventObject)
	 */
	public void beforeSupercedeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSupercedeEvent(java.util.EventObject)
	 */
	public void afterSupercedeEvent(EventObject e) throws RecordManagerException {
		logger.info("afterSupercedeEvent - BEGIN");
		
		//retrieve the parameters
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);

		long supercededFPCId = ((Long) map.get(ExtensionParamKeys.SUPERCEDED_FILE_PLAN_COMPONENT_ID)).longValue();
		logger.info("supercededFPCId " + supercededFPCId);
		
		long supercedingFPCId = ((Long) map.get(ExtensionParamKeys.SUPERCEDING_FILE_PLAN_COMPONENT_ID)).longValue();
		logger.info("supercedingFPCId " + supercedingFPCId);
		
		Date supercedeDate = (Date) map.get(ExtensionParamKeys.SUPERCEDE_DATE);
		logger.info("supercedeDate " + supercedeDate);

		long viewId = ((Long) map.get(ExtensionParamKeys.VIEW_ID)).longValue();
		logger.info("viewId " + viewId);

		boolean isLifeCycleBegin = ((Boolean) map.get(ExtensionParamKeys.IS_LIFECYCLE_BEGIN)).booleanValue();
		logger.info("isLifeCycleBegin " + isLifeCycleBegin);
		
		//get full path of the superceded fpc and superceding fpc.
		NavigationControllerEJBLocalHome nch = null;
		NavigationControllerEJBLocal ncl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.NAVIGATION_CONTROLLER_JNDI);
	
			nch = (NavigationControllerEJBLocalHome)PortableRemoteObject.narrow(obj, NavigationControllerEJBLocalHome.class);
	
			ncl = nch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		String supercededFPCFullPathXML = ncl.getFullPathListById(supercededFPCId, DualNamingOptions.BY_NAME);
		String supercedingFPCFullPathXML = ncl.getFullPathListById(supercedingFPCId, DualNamingOptions.BY_NAME);
		
		Document supercededFPCFullPathDoc = XMLUtil.getDocument(supercededFPCFullPathXML);
		Document supercedingFPCFullPathDoc = XMLUtil.getDocument(supercedingFPCFullPathXML);
		String supercededFPCFullPath = XMLUtil.getStringValue(
				XMLUtil.getNodeByXPath(supercededFPCFullPathDoc,"/List/Item/@FullPath"));
		String supercedingFPCFullPath = XMLUtil.getStringValue(
				XMLUtil.getNodeByXPath(supercedingFPCFullPathDoc,"/List/Item/@FullPath"));
		
		//get the life cycle code of the superceded fpc
		FilePlanComponentControllerEJBLocalHome fch =null;
		FilePlanComponentControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.FILE_PLAN_COMPONENT_CONTROLLER_JNDI);
	
			fch = (FilePlanComponentControllerEJBLocalHome) PortableRemoteObject.narrow(obj, FilePlanComponentControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		
		String lccField = fcl.getFilePlanComponentField(supercededFPCId,"LfCclCdID");
		
		if(ValidationUtil.isEmpty(lccField) || lccField.equals("0")){
			logger.info("Life Cycle Code of the superceded FPC is empty or 0. No E-mail is sent.");
			logger.info("afterSupercedeEvent - END");
			return;
		}
		long codeId = Long.parseLong(lccField);
		
		//get disposal type and beginLifeCycleWhenSuperceded
		LifeCycleDesignControllerEJBLocalHome lcdch = null;
		LifeCycleDesignControllerEJBLocal lcdcl =null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.LIFE_CYCLE_DESIGN_CONTROLLER_JNDI);
	
			lcdch = (LifeCycleDesignControllerEJBLocalHome) PortableRemoteObject.narrow(obj, LifeCycleDesignControllerEJBLocalHome.class);
	
			lcdcl = lcdch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		
		String lifeCycleXML = lcdcl.getLifeCycleManagementCodeById(codeId);		
		Document lifeCycleDoc = XMLUtil.getDocument(lifeCycleXML);
		
		String supercedeValue = XMLUtil.getNodeValue(lifeCycleDoc, "Field", "FlPlnCmpntAttrDefColNm", 
									"BgnLfCclWhenSprsd");
		String disposalTypeValue = XMLUtil.getNodeValue(lifeCycleDoc, "Field", "FlPlnCmpntAttrDefColNm", 
									"DspTyp");

		boolean isBeginLCCWhenSuperceded =false;
		if (supercedeValue !=null && supercedeValue.equals("1")) {
			isBeginLCCWhenSuperceded = true;
		}
		
		long disposalType = Long.parseLong(disposalTypeValue);
		
		if (isBeginLCCWhenSuperceded && disposalType == DispositionTypes.DESTROY) {
			
			
			initApplicationResources();
			String userQuery = buildUsersQuery(cstEmailAddress,cstInformOnSupercede);
			loginAsAdministrator(stateMachine);
			
			//get users to send email to
			QueryControllerEJB qc = getQueryController(); 
			
			String userResult = null;
			try { 
				userResult = qc.runQuery(userQuery, QueryResultOptions.NORMAL);
			} catch(Exception exp) {
				throw new RecordManagerException(BusinessLayerError.UNKNOWN, exp.getMessage(),getClass().getName(),
					RecordManagerExceptionTypes.SYSTEM, exp);
			}

			Document domDoc = XMLUtil.getDocument(userResult);	
			NodeList nodeList = XMLUtil.getNodeListByXPath(domDoc,"//ResultList/Data/List/Item");
			String [] emailAddress ;
			int nLen = nodeList.getLength();
			emailAddress = new String[nLen];

			String usrCstEmailAddress = "TS_UsrDir_Cst." + cstEmailAddress;
			for (int i = 0; i < nLen ; i++){
				Node nodeItem = nodeList.item(i);
				emailAddress[i] = XMLUtil.getStringValueFromXMLAttribute(nodeItem,usrCstEmailAddress);
			}
			
			StringBuffer body = new StringBuffer();
			body.append(supercededFPCFullPath).append(" (id:").append(supercededFPCId).append(")").
				append(" is superceded in a category that has a final disposition of " +
					"\"Destroy when superseded or obsolete.\" by ").
				append(supercedingFPCFullPath).append(" (id:").append(supercedingFPCId).append(") ");
			
			if (emailAddress.length==0) {
				logger.info("No E-mail address is configured. No E-mail is sent.");
				logger.info("This Message should have sent if E-mail address is configured:"+ body.toString());
				logger.info("afterSupercedeEvent - END");
				return;
			}
			
			sendEmail(mailFromAddress, emailAddress, "Supercede", body.toString());
		}
		

		logger.info("afterSupercedeEvent - END");
	}

	private String buildUsersQuery (String userEmailAddress, String userInformOnSupercede) {
		StringBuffer sbQuery = new StringBuffer();
		// -3 for user
		sbQuery.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		sbQuery.append("<Fields><Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntDefID\">-3</Field>");
		sbQuery.append("<Field FlPlnCmpntAttrDefColNm=\"QueryImage\">");
		// ReportTemplate - Start
		sbQuery.append("<![CDATA[<ReportTemplate>");
		// parameters
		sbQuery.append("<Version>1.0</Version>");
		sbQuery.append("<ReportLayout />");
		sbQuery.append("<CountOnly>0</CountOnly>");
		sbQuery.append("<Scope />"); 
		sbQuery.append("<BlockSize>0</BlockSize>");
		sbQuery.append("<BlockStart>0</BlockStart>");
  		
		// Query - Start
		sbQuery.append("<Query>");
  		
		// Condition - Start
		sbQuery.append("<Condition>");
  		
  		
		sbQuery.append("<FieldExpression>");
		sbQuery.append("<FieldName>TS_UsrDir_Cst." + userInformOnSupercede + "</FieldName>");
		sbQuery.append("<FieldOperator>=</FieldOperator>");
		sbQuery.append("<FieldValue>1</FieldValue>");
		sbQuery.append("</FieldExpression>");
  		
		sbQuery.append("<InterFieldOperator>AND</InterFieldOperator>");
  		
		sbQuery.append("<InterFieldOperator>NOT</InterFieldOperator>");
		sbQuery.append("<FieldExpression>");
		sbQuery.append("<FieldName>TS_UsrDir_Cst." + userEmailAddress + "</FieldName>");
		sbQuery.append("<FieldOperator>IS NULL</FieldOperator>");
		sbQuery.append("<FieldValue/>");
		sbQuery.append("</FieldExpression>");  		
    		
		// Condition - End
		sbQuery.append("</Condition>");  
		// SortFields
		sbQuery.append("<SortFields />");
		// OutputFields - Begin
		sbQuery.append("<OutputFields>");
		sbQuery.append("<OutputField>TS_UsrDir_Cst." + userEmailAddress + "</OutputField>");
		sbQuery.append("<OutputField>TS_UsrDir.UsrNm</OutputField>");
		// OutputFields - End
		sbQuery.append("</OutputFields>");
  		
		// Query - End
		sbQuery.append("</Query>");
  		  		
		// ReportTemplate - End		
		sbQuery.append("</ReportTemplate>]]>");
		sbQuery.append("</Field></Fields>");
		
		return sbQuery.toString();		
	}



	/**
	 * 
	 */
	private void sendEmail(String fromAddress,String[] toAddress, String subject, String body) 
		throws RecordManagerException {
		Properties props = new Properties ();
		props.put("mail.smtp.host", mailSmtpHost);
		logger.info("smtp host:" + mailSmtpHost);   
		try {
			
			Session sendMailSession = Session.getDefaultInstance(props);
	   		
			MimeMessage mailMessage = new MimeMessage(sendMailSession);
			mailMessage.setSubject(subject);
	
			mailMessage.setText(body);
	
	
			InternetAddress addressFrom = new InternetAddress(fromAddress);
			mailMessage.setFrom(addressFrom);
	   		
			for (int i=0 ; i < toAddress.length ; i++){
				mailMessage.addRecipient(RecipientType.TO ,new InternetAddress(toAddress[i]));
			}
			java.util.Enumeration enum1 = mailMessage.getAllHeaderLines();
			while(enum1.hasMoreElements()){
				Object obj = enum1.nextElement();
				logger.info(obj.toString());
			}
			
			Transport.send(mailMessage);
			
			
		} catch (Exception e) {
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, e.getMessage(), 
				getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
		logger.info("E-mail is sent:"+body);
		
	}

	/**
	 * Method initApplicationResources.
	 * This function must be called before all events. It will initialize requires login/pw
	 * project and suppMark from ApplicationResources.
	 * 
	 */
	private void initApplicationResources () {
		logger.info("initApplicationResources - BEGIN");
		InputStream is = DocumentListener.class.getResourceAsStream(APP_RES_FILE_NAME);
		prop = new Properties();
		if(null != is){
			try{
				prop.load(is);
			}
			catch(IOException e){
				logger.info("initApplicationResources - Properties.load failed ",e);
			}
		}
		else{
			logger.info("initApplicationResources - getResoucesAsStream is null");
		}
		adminUserName = prop.getProperty(APP_RES_LOGIN_NAME,ADMIN_USER_NAME);
		adminPassword = prop.getProperty(APP_RES_LOGIN_PW,ADMIN_PASSWORD);
		
		cstEmailAddress = prop.getProperty(APP_RES_CST_EMAIL_ADDRESS);
		cstInformOnSupercede = prop.getProperty(APP_RES_CST_INFORM_ON_SUPERCEDE);
		mailSmtpHost = prop.getProperty(APP_RES_MAIL_SMTP_HOST);
		mailFromAddress = prop.getProperty(APP_RES_MAIL_FROM_ADDRESS);
		
		transFolderDefinitionName =	prop.getProperty(APP_TRANS_FOLDER_DEF_NAME);              
		transFolderLifeCycleCodeName = prop.getProperty(APP_TRANS_FOLDER_LFC_NAME);     
		transFolderLifeCycleCodeInstructionName = prop.getProperty(APP_TRANS_FOLDER_LFC_INSTRUCTION_NAME);
		transFolderLifeCyclePhaseName = prop.getProperty(APP_TRANS_FOLDER_LFC_PHS_NAME);   
		transRecordDefinitionName = prop.getProperty(APP_TRANS_RECORD_DEF_NAME);
		transLinkViewName = prop.getProperty(APP_TRANS_LNK_VW_NAME);               
		transLinkedRecordDefinitionName = prop.getProperty(APP_TRANS_LNKED_RECORD_DEF_NAME);
		
		transExportSchemaName = prop.getProperty(APP_TRAN_EXOPT_SCHEMA_NAME);
		transExportCategoryName = prop.getProperty(APP_TRAN_EXOPT_CAT_NAME);          
		transExportCategoryNARAIndicator = prop.getProperty(APP_TRAN_EXOPT_CAT_NARA_INDICATOR);
		transExportCategoryPermanentIndicator = prop.getProperty(APP_TRAN_EXOPT_CAT_PERMANENT_INDICATOR);
		transExportPermanentDir = (String) prop.get(APP_TRAN_EXOPT_PERM_DIR);
		transExportTemporaryDir = (String) prop.get(APP_TRAN_EXOPT_TEMP_DIR);
		
		declassifyLifeCycleCodeName = prop.getProperty(APP_DECA_LFC_NAME);
		declassifyLifeCycleNeedReviewAttributeName = prop.getProperty(APP_DECA_LFC_REV_ATTR_NAME);
		declassifyLifeCycleInstructionName = prop.getProperty(APP_DECA_LFC_INSTRUCTION_NAME);
		declassifyLifeCyclePhaseName = prop.getProperty(APP_DECA_LFC_PHS_NAME);
		declassifyFolderName = prop.getProperty(APP_DECA_FOLDER_NAME);
//		declassifyFolderReviewDoneAttibuteName = prop.getProperty(APP_DECA_FOLDER_REV_DONE_ATTR_NAME);
		declassifyRecordName = prop.getProperty(APP_DECA_RECORD_NAME);
		declassifyEmailAttrName = prop.getProperty(APP_DECA_EMAIL_ATTR_NAME);
		declassifyAdminEmail = prop.getProperty(APP_DECA_ADMIN_EMAIL);
		
		logger.info("initApplicationResources - END");		
	}
	private String getMessage(String key){
		return prop.getProperty(key,key);
	}
	private String getMessage(String key, String arg0){
		return getMessage (key,arg0,null,null);
	}	
	private String getMessage(String key, String arg0, String arg1){
		return getMessage(key,arg0,arg1,null);
	}
	private String getMessage(String key, String arg0, String arg1, String arg2){
		String msg = prop.getProperty(key,key);
		String formatedMsg ;
		try{
			String [] args = {arg0, arg1, arg2};
			formatedMsg = MessageFormat.format(msg,args);
		}
		catch(IllegalArgumentException e){
			formatedMsg = key;
		}
		return formatedMsg;
	}	
	/**
	 * creates adminClientController for permission modification
	 * 
	 * Method loginAsAdministrator.
	 * @param stateMachine
	 * @throws RecordManagerException
	 */
	private void loginAsAdministrator(StateMachine stateMachine) throws RecordManagerException{

		try{
			logger.info("loginAsAdministrator - Before getLoginManagerEJBHome ...");
			if(null == context){
				context = new InitialContext ();
			}
			Object obj = context.lookup(BusinessComponentsJNDINames.LOGIN_MANAGER_EJB_JNDI);
			LoginManagerEJBHome loginMgrHome = (LoginManagerEJBHome) PortableRemoteObject.narrow(obj, LoginManagerEJBHome.class);

//			logger.info("loginAsAdministrator - after getLoginManagerEJBHome ...");
						
			LoginManagerEJB loginMgr = loginMgrHome.create();
//			logger.info("loginAsAdministrator - Before getDatasource ...");
			DBConfig dbConfig = (DBConfig)stateMachine.getAttribute(StateMachineKeys.DATASOURCE);
			logger.info("loginAsAdministrator - Before login Datasource [" + dbConfig.jndi_name + "] User [" + adminUserName + "]");
			adminClientController = loginMgr.login(adminUserName,adminPassword,dbConfig.jndi_name,"","en");
			loginMgr.remove();
			logger.info("loginAsAdministrator - DONE! ...");
		}
		catch(Exception e){
			logger.info("loginAsAdministrator - caught exception!",e);
			if(e instanceof RecordManagerException ){
				RecordManagerException rme = (RecordManagerException) e;
				if(DAOLayerErrors.USER_NOT_FOUND == rme.getErrorKey()){
					throw new RecordManagerException(E_EXTSN_LOGIN_FAILED, getMessage(E_RES_LOGIN_BAD_USR_PW), getClass().getName(), RecordManagerExceptionTypes.SYSTEM,rme);					
				}
				throw rme;
			}
			else{
				throw new RecordManagerException(E_EXTSN_NO_CLNT_CNTRLR, e.getMessage(), this.getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
			}
		}
	}	

	private QueryControllerEJB getQueryController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			QueryControllerEJB qc = adminClientController.getQueryController();
			return qc;			
		}
		catch(Exception e){
			logger.info ("getUserController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}			
	
	private DefinitionControllerEJB getDefinitionController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			DefinitionControllerEJB dc = adminClientController.getDefinitionController();
			return dc;			
		}
		catch(Exception e){
			logger.info ("getDefinitionController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}		
	
	private DefinitionControllerEJBLocal getDefinitionControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		DefinitionControllerEJBLocalHome fch =null;
		DefinitionControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.DEFINITION_CONTROLLER_JNDI);
	
			fch = (DefinitionControllerEJBLocalHome) PortableRemoteObject.narrow(obj, DefinitionControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}
	
	private UserControllerEJBLocal getUserControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		UserControllerEJBLocalHome fch =null;
		UserControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.USER_CONTROLLER_JNDI);
	
			fch = (UserControllerEJBLocalHome) PortableRemoteObject.narrow(obj, UserControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}
	
	private FilePlanComponentControllerEJB getFilePlanComponentController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			FilePlanComponentControllerEJB fc = adminClientController.getFilePlanComponentController();
			return fc;			
		}
		catch(Exception e){
			logger.info ("getFilePlanComponentController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}
	
	private FilePlanComponentControllerEJBLocal getFilePlanComponentControllerLocal(StateMachine stateMachine) throws RecordManagerException{
		FilePlanComponentControllerEJBLocalHome fch =null;
		FilePlanComponentControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.FILE_PLAN_COMPONENT_CONTROLLER_JNDI);
	
			fch = (FilePlanComponentControllerEJBLocalHome) PortableRemoteObject.narrow(obj, FilePlanComponentControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}
	
	private LifeCycleDesignControllerEJB getLifeCycleDesignController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			LifeCycleDesignControllerEJB lc = adminClientController.getLifeCycleDesignController();
			return lc;			
		}
		catch(Exception e){
			logger.info ("getLifeCycleDesignController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}	
	
	private LifeCycleDesignControllerEJBLocal getLifeCycleDesignControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		LifeCycleDesignControllerEJBLocalHome fch =null;
		LifeCycleDesignControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.LIFE_CYCLE_DESIGN_CONTROLLER_JNDI);
	
			fch = (LifeCycleDesignControllerEJBLocalHome) PortableRemoteObject.narrow(obj, LifeCycleDesignControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}		
	
	private NavigationControllerEJB getNavigationController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			NavigationControllerEJB nc = adminClientController.getNavigationController();
			return nc;			
		}
		catch(Exception e){
			logger.info ("getNavigationController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}	
	
	private NavigationControllerEJBLocal getNavigationControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		NavigationControllerEJBLocalHome fch =null;
		NavigationControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.NAVIGATION_CONTROLLER_JNDI);
	
			fch = (NavigationControllerEJBLocalHome) PortableRemoteObject.narrow(obj, NavigationControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}			
		
	private FilePlanDesignControllerEJB getFilePlanDesignController() throws RecordManagerException{
		try{
			// need queryController with Administrator because we need call function that requires Security Management function access
			FilePlanDesignControllerEJB fc = adminClientController.getFilePlanDesignController();
			return fc;			
		}
		catch(Exception e){
			logger.info ("getFilePlanDesignController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}	
	
	private FilePlanDesignControllerEJBLocal getFilePlanDesignControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		FilePlanDesignControllerEJBLocalHome fch =null;
		FilePlanDesignControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.FILE_PLAN_DESIGN_CONTROLLER_JNDI);
	
			fch = (FilePlanDesignControllerEJBLocalHome) PortableRemoteObject.narrow(obj, FilePlanDesignControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}
	
	private QueryControllerEJBLocal getQueryController(StateMachine stateMachine) throws RecordManagerException{
		QueryControllerEJBLocalHome qlh= null;
		QueryControllerEJBLocal ql= null;		
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.QUERY_CONTROLLER_JNDI);
	
			qlh = (QueryControllerEJBLocalHome) PortableRemoteObject.narrow(obj, QueryControllerEJBLocalHome.class);
	
			ql = qlh.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return ql;
	}		
	
	private SchemaMappingControllerEJBLocal getSchemaMappingControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		SchemaMappingControllerEJBLocalHome fch =null;
		SchemaMappingControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.SCHEMA_MAPPING_CONTROLLER_JNDI);
	
			fch = (SchemaMappingControllerEJBLocalHome) PortableRemoteObject.narrow(obj, SchemaMappingControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}	
	
	private ConfigurationControllerEJBLocal getConfigurationControllerEJBLocal(StateMachine stateMachine) throws RecordManagerException{
		ConfigurationControllerEJBLocalHome fch =null;
		ConfigurationControllerEJBLocal fcl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.CONFIGURATION_CONTROLLER_JNDI);
	
			fch = (ConfigurationControllerEJBLocalHome) PortableRemoteObject.narrow(obj, ConfigurationControllerEJBLocalHome.class);
	
			fcl = fch.create(stateMachine);			

		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}
		return fcl;
	}			

	public void afterTransitionExportToSpecificSchemaEvent(EventObject e) throws RecordManagerException {
		logger.info("afterTransitionExportToSpecificSchema - BEGIN");
		this.initApplicationResources();		
		
		HashMap permanentCategories = new HashMap();
		HashMap temporaryCategories = new HashMap();
		
		//retrieve the parameters
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		String exportSchemaType = (String) map.get(ExtensionParamKeys.TRANSITION_EXPORT_SCHEMA_TYPE);		
		String exportSchemaDirectory = (String) map.get(ExtensionParamKeys.TRANSITION_EXPORT_SCHEMA_DIRECTORY);	
		String[] exportSchemaFiles = (String[]) map.get(ExtensionParamKeys.TRANSITION_EXPORT_SCHEMA_FILES);
		
		SchemaMappingControllerEJBLocal schemaBean = this.getSchemaMappingControllerEJBLocal(stateMachine);
		String schemaMapping = null;
		try {
			schemaMapping = schemaBean.getSchemaMappingByName(this.transExportSchemaName);
		} catch (RemoteException e1) {
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, e1.getMessage(),getClass().getName(),
					RecordManagerExceptionTypes.SYSTEM, e1);
		}
		if(schemaMapping == null) return;
		Document schemaMappingDoc = XMLUtil.getDocument(schemaMapping);
		String transExportSchemaId = XMLUtil.getNodeValue(schemaMappingDoc, "Field", "FlPlnCmpntAttrDefColNm", "MapID");
		if(transExportSchemaId == null || transExportSchemaId.equals("")) return;
						
		if(exportSchemaType == null || ! exportSchemaType.equals(transExportSchemaId) ||
				exportSchemaDirectory == null || 
				exportSchemaFiles == null || 
				exportSchemaFiles.length == 0) return;
		
		File permanentDir = new File(exportSchemaDirectory+fileSeparator+transExportPermanentDir);
		if(! permanentDir.exists() || ! permanentDir.isDirectory()){
			permanentDir.mkdir();
		}
		File temporaryDir = new File(exportSchemaDirectory+fileSeparator+transExportTemporaryDir);
		if(! temporaryDir.exists() || ! temporaryDir.isDirectory()){
			temporaryDir.mkdir();
		}
		
		//get all APIs
		ConfigurationControllerEJBLocal configBean = this.getConfigurationControllerEJBLocal(stateMachine);
		String  pathDelimiter= configBean.getEngineConfigValue(com.ibm.gre.engine.util.EngineConfigKeys.FULLPATH_LEVEL_DELIMITER, 0);
		DefinitionControllerEJBLocal defBean = this.getDefinitionControllerEJBLocal(stateMachine);
		String fpd = defBean.getComponentDefinitionByName(this.transExportCategoryName);
		Document fpdDoc = XMLUtil.getDocument(fpd);
		Node fpdNode = XMLUtil.getNodeByXPath(fpdDoc, "/List/Item");
		long categoryDefId = XMLUtil.getLongValueFromXMLAttribute(fpdNode, "FlPlnCmpntDefID");
		String categoryCustTblNm = XMLUtil.getStringValueFromXMLAttribute(fpdNode, "CustTblNm");
		String categoryQuery = this.buildCategoryQuery(categoryDefId, categoryCustTblNm);
		
		QueryControllerEJBLocal qc = getQueryController(stateMachine); 
		
		String categoryResult = null;
		try { 
			categoryResult = qc.runQuery(categoryQuery, QueryResultOptions.NORMAL);
		} catch(Exception exp) {
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, exp.getMessage(),getClass().getName(),
				RecordManagerExceptionTypes.SYSTEM, exp);
		}
		
		Document domDoc = XMLUtil.getDocument(categoryResult);
		NodeList nodeList = XMLUtil.getNodeListByXPath(domDoc,"/ResultList/Data/List/Item");
		String permanentIndicatorAttr = categoryCustTblNm +"."+ this.transExportCategoryPermanentIndicator;
		String pathAttr = "NameFullPath";		
		String nameAttr = "TS_FlPlnCmpnt.FlPlnCmpntNm";
		for (int i = 0; i < nodeList.getLength() ; i++){
			Node nodeItem = nodeList.item(i);
			String name = XMLUtil.getStringValueFromXMLAttribute(nodeItem,nameAttr);
			String path = XMLUtil.getStringValueFromXMLAttribute(nodeItem,pathAttr);
			long permnt = XMLUtil.getLongValueFromXMLAttribute(nodeItem, permanentIndicatorAttr);
			
			if(permnt == 1) 
				permanentCategories.put(path, name);
			else
				temporaryCategories.put(path, name);
			
		}
		
		for(int i = 0;i <exportSchemaFiles.length; i++){
			seperateToFile(exportSchemaDirectory, exportSchemaFiles[i], permanentCategories, this.transExportPermanentDir,pathDelimiter);
			seperateToFile(exportSchemaDirectory, exportSchemaFiles[i], temporaryCategories, this.transExportTemporaryDir,pathDelimiter);
		}
		
		logger.info("afterTransitionExportToSpecificSchema - END");
	}
	
	private void seperateToFile(String exportSchemaDirectory, String fileName, HashMap categories, String exportDir, String pathDelimiter) throws RecordManagerException {		
		String inFullPath = fileName;
		InputSource inSource = null;
		try {
			inSource = new InputSource(new BufferedReader(
						new InputStreamReader(new FileInputStream(inFullPath),"UTF-8")));
		} catch(FileNotFoundException e) {
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, e.getMessage(), DocumentListener.class.getName(), RecordManagerExceptionTypes.SYSTEM, e);
		} catch(UnsupportedEncodingException e) {
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, e.getMessage(), DocumentListener.class.getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
		
		Document fpcDoc = XMLUtil.getDocument(inSource);		
		boolean found = false;
		Node folderNode = XMLUtil.getNodeByXPath(fpcDoc, "/Folder");
		
		if(folderNode != null){
			String categoryIdentifier = XMLUtil.getOptionalStringValueFromAttribute(folderNode, "RecordCategoryIdentifier");
			if(categoryIdentifier != null && !categoryIdentifier.equals("")){
				if(categories.get(categoryIdentifier) != null){
					found = true;
				}
			}
		}else{
			Node recordNode = XMLUtil.getNodeByXPath(fpcDoc, "/Record");
			if(recordNode != null){
				String folderIdentifier = XMLUtil.getOptionalStringValueFromAttribute(recordNode, "FolderIdentifier");
				int l = folderIdentifier.lastIndexOf(pathDelimiter);
				if(l>0){
					String categoryIdentifier = folderIdentifier.substring(0, l);
					if(categories.get(categoryIdentifier) != null){
						found = true;
					}					
				}
			}
			
		}
		
		if(found == true){
			String rawFileName = null;
			int pos = fileName.lastIndexOf(fileSeparator);
			if (pos != -1){
				if(pos == fileName.length() - 1) return;
				rawFileName = fileName.substring( pos );
			}else{
				rawFileName = fileName;
			}
			writeXML(fpcDoc, exportSchemaDirectory +fileSeparator +exportDir +fileSeparator +rawFileName);		
		}
		
	}
	
	public static void writeXML(Document doc, String fileName) throws RecordManagerException 
	{    		   	
		try{
			Element element = doc.getDocumentElement();
		   	Writer writer = new OutputStreamWriter(new FileOutputStream(fileName));

		   	OutputFormat of = new OutputFormat(Method.XML,"UTF-8",false);
			of.setLineSeparator("\n");	
							
			XMLSerializer serializer = new XMLSerializer(writer,of);
			serializer.serialize(element);
			writer.close();
		}
		catch (Exception e){
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, e.getMessage(), DocumentListener.class.getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}			
	 }	
	
	private String buildCategoryQuery (long categoryDef, String categoryCstTableName) {
		String queryImage = " <ReportTemplate> "+
							" 	<Version>8.5.0.000</Version> "+
							" 	<ReportLayout /> "+ 
							" 	<Scope />  "+
							" 	<BlockSize>0</BlockSize>  "+
							" 	<BlockStart>0</BlockStart>  "+
							" 	<Distinct>0</Distinct>  "+
							" 	<Query> "+
							" 		<Condition> "+
							" 			<FieldExpression> "+
							" 				<FieldName>"+categoryCstTableName+"."+transExportCategoryNARAIndicator+"</FieldName>  "+
							" 				<FieldOperator>=</FieldOperator>  "+
							" 				<FieldValue>1</FieldValue>  "+
							" 			</FieldExpression> "+
							" 		</Condition> "+
							" 		<SortFields />  "+
							" 		<OutputFields> "+
							" 			<OutputField>TS_FlPlnCmpnt.FlPlnCmpntNm</OutputField>  "+
							" 			<OutputField>TS_FlPlnCmpnt.FlPlnCmpntCode</OutputField>  "+
							" 			<OutputField>TS_FlPlnCmpnt.FlPlnCmpntTtl</OutputField>  "+
							" 			<OutputField>NameFullPath</OutputField>  "+
							" 			<OutputField>"+categoryCstTableName+"."+transExportCategoryNARAIndicator+"</OutputField>  "+
							" 			<OutputField>"+categoryCstTableName+"."+transExportCategoryPermanentIndicator+"</OutputField>  "+
							" 		</OutputFields> "+
							" 	</Query> "+
							" </ReportTemplate> ";
		
		String query = "<Fields> "+
					   " <Field FlPlnCmpntAttrDefColNm=\"FlPlnCmpntDefID\">"+categoryDef+"</Field>  "+
					   " <Field FlPlnCmpntAttrDefColNm=\"QueryImage\"> "+
					   "  <![CDATA[ "+queryImage+" ]]>  "+
					   " </Field> "+
					   "</Fields> ";
		
		return query;		
	}	
}
